# coding=utf-8

"""

This is a slightly unusual 'collector'. It collects and uploads stats to a
app engine instance run by Rob Smith. The stats collected are:

 * Per collector enabled or not
 * Per collector interval time
 * Global collector reload interval
 * Global handlers
 * Any custom collector set stats

These stats are stored anonymously (other then UUID), processed and the results
are at [http://diamond-stats.appspot.com/](http://diamond-stats.appspot.com/).

These values can help us know more about how Diamond is being used and can help
us target development efforts in the future.

#### Requirements

You can install Smolt and run it once or run the following from a terminal

Linux:

 * mkdir /etc/smolt
 * cd /etc/smolt
 * cat /proc/sys/kernel/random/uuid > hw-uuid

Others:

 * mkdir /etc/smolt
 * cd /etc/smolt
 * curl -Lo- http://utils.kormoc.com/uuid/get.php > hw-uuid

#### Dependencies

 * /etc/smolt/hw-uuid
 * urllib
 * json/simplejson

"""

from diamond.collector import Collector
from diamond.util import get_diamond_version
import diamond

import urllib
import os
import sys
import platform

try:
    import json
    json  # workaround for pyflakes issue #13
except ImportError:
    import simplejson as json


def getIncludePaths(path):
    for f in os.listdir(path):
        cPath = os.path.abspath(os.path.join(path, f))

        if os.path.isfile(cPath) and len(f) > 3 and f[-3:] == '.py':
            sys.path.append(os.path.dirname(cPath))

    for f in os.listdir(path):
        cPath = os.path.abspath(os.path.join(path, f))
        if os.path.isdir(cPath):
            getIncludePaths(cPath)

collectors = {}


def getCollectors(path):
    for f in os.listdir(path):
        cPath = os.path.abspath(os.path.join(path, f))

        if (os.path.isfile(cPath)
                and len(f) > 3
                and f[-3:] == '.py'
                and f[0:4] != 'test'):
            modname = f[:-3]
            try:
                # Import the module
                module = __import__(modname, globals(), locals(), ['*'])

                # Find the name
                for attr in dir(module):
                    cls = getattr(module, attr)
                    try:
                        if (issubclass(cls, Collector)
                                and cls.__name__ not in collectors):
                            collectors[cls.__name__] = module
                            break
                    except TypeError:
                        continue
            except Exception:
                collectors[modname] = False
                continue

    for f in os.listdir(path):
        cPath = os.path.abspath(os.path.join(path, f))
        if os.path.isdir(cPath):
            getCollectors(cPath)


class StatsCollector(diamond.collector.Collector):
    full_config = None

    def __init__(self, config, handlers):
        self.full_config = config
        super(StatsCollector, self).__init__(config, handlers)

    def get_default_config_help(self):
        config_help = super(StatsCollector, self).get_default_config_help()
        config_help.update({
            'url': 'The url to post stats to.',
            'uuidfile': 'The path to the uuid file'
        })
        return config_help

    def get_default_config(self):
        """
        Returns the default collector settings
        """
        config = super(StatsCollector, self).get_default_config()
        config.update({
            'url': 'http://diamond-stats.appspot.com/submitstats',
            'uuidfile': '/etc/smolt/hw-uuid',
            'interval': 604800,
        })
        return config

    def collect(self):
        getIncludePaths(self.full_config['server']['collectors_path'])
        getCollectors(self.full_config['server']['collectors_path'])

        stats = {}

        stats['version'] = get_diamond_version()
        stats['python_version'] = platform.python_version()

        if platform.system() == 'Darwin':
            ver = platform.mac_ver()
            os_version = ('Darwin', ver[0], '')

        elif platform.system() == 'Linux':
            os_version = platform.linux_distribution()

        elif platform.system() == 'Windows':
            ver = platform.win32_ver()
            os_version = ('Windows', ver[0], ver[2])

        stats['os'] = "%s %s" % (os_version[0], os_version[1])
        stats['os_distname'] = os_version[0]
        stats['os_version'] = os_version[1]
        stats['os_id'] = os_version[2]

        uuid_file = open(self.config['uuidfile'])
        stats['uuid'] = uuid_file.read().strip()
        uuid_file.close()

        stats['collectors'] = {}

        stats['collectors']['Default'] = self.get_stats_for_upload(
            config=self.full_config['collectors']['default'])

        for collector in collectors:
            if not hasattr(collectors[collector], collector):
                continue

            cls = getattr(collectors[collector], collector)
            obj = cls(config=self.full_config, handlers={})
            stats['collectors'][collector] = obj.get_stats_for_upload()

        stats['collectors']['StatsCollector'] = self.get_stats_for_upload()

        stats['server'] = {}

        if type(self.full_config['server']['handlers']) is list:
            handlers = self.full_config['server']['handlers']
        else:
            handlers = self.full_config['server']['handlers'].split(',')
        stats['server']['handlers'] = handlers

        reload_i = self.full_config['server']['collectors_reload_interval']
        stats['server']['collectors_reload_interval'] = reload_i

        hmeth = 'Default'
        if 'hostname_method' in self.full_config['collectors']['default']:
            hmeth = self.full_config['collectors']['default']['hostname_method']
        stats['server']['hostname_method'] = hmeth

        data = urllib.urlencode({'stats': json.dumps(stats)})
        f = urllib.urlopen(self.config['url'], data)
        f.read()
        f.close()

        return True



# coding=utf-8

"""
Collects the following from beanstalkd:
    - Server statistics via the 'stats' command
    - Per tube statistics via the 'stats-tube' command

#### Dependencies

 * beanstalkc

"""

import re
import diamond.collector

try:
    import beanstalkc
    beanstalkc  # workaround for pyflakes issue #13
except ImportError:
    beanstalkc = None


class BeanstalkdCollector(diamond.collector.Collector):
    COUNTERS_REGEX = re.compile(
        r'^(cmd-.*|job-timeouts|total-jobs|total-connections)$')

    def get_default_config_help(self):
        config_help = super(BeanstalkdCollector,
                            self).get_default_config_help()
        config_help.update({
            'host': 'Hostname',
            'port': 'Port',
        })
        return config_help

    def get_default_config(self):
        """
        Returns the default collector settings
        """
        config = super(BeanstalkdCollector, self).get_default_config()
        config.update({
            'path':     'beanstalkd',
            'host':     'localhost',
            'port':     11300,
        })
        return config

    def _get_stats(self):
        stats = {}
        try:
            connection = beanstalkc.Connection(self.config['host'],
                                               int(self.config['port']))
        except beanstalkc.BeanstalkcException, e:
            self.log.error("Couldn't connect to beanstalkd: %s", e)
            return {}

        stats['instance'] = connection.stats()
        stats['tubes'] = []

        for tube in connection.tubes():
            tube_stats = connection.stats_tube(tube)
            stats['tubes'].append(tube_stats)

        return stats

    def collect(self):
        if beanstalkc is None:
            self.log.error('Unable to import beanstalkc')
            return {}

        info = self._get_stats()

        for stat, value in info['instance'].items():
            if stat != 'version':
                self.publish(stat, value,
                             metric_type=self.get_metric_type(stat))

        for tube_stats in info['tubes']:
            tube = tube_stats['name']
            for stat, value in tube_stats.items():
                if stat != 'name':
                    self.publish('tubes.%s.%s' % (tube, stat), value,
                                 metric_type=self.get_metric_type(stat))

    def get_metric_type(self, stat):
        if self.COUNTERS_REGEX.match(stat):
            return 'COUNTER'
        return 'GAUGE'



#!/usr/bin/python
# coding=utf-8

from test import CollectorTestCase
from test import get_collector_config
from test import unittest
from mock import MagicMock, Mock
from mock import patch

from diamond.collector import Collector
from mountstats import MountStatsCollector


class TestMountStatsCollector(CollectorTestCase):
    def setUp(self):
        config = get_collector_config('MountStatsCollector', {
            'exclude_filters': ['^/mnt/path2'],
            'interval': 1
        })

        self.collector = MountStatsCollector(config, None)

    def test_import(self):
        self.assertTrue(MountStatsCollector)

    @patch('__builtin__.open')
    @patch('os.access', Mock(return_value=True))
    @patch.object(Collector, 'publish')
    def test_should_open_mountstats(self, publish_mock, open_mock):
        open_mock.return_value = MagicMock()
        self.collector.collect()
        open_mock.assert_called_once_with(self.collector.MOUNTSTATS)

    @patch.object(Collector, 'publish')
    def test_should_work_with_real_data(self, publish_mock):
        # Test the first and last metric of each type
        published_metrics = {
            '_mnt_path1.events.inoderevalidates': 27110.0,
            '_mnt_path1.events.delay': 0.0,
            '_mnt_path1.bytes.normalreadbytes': 1424269.0,
            '_mnt_path1.bytes.serverwritebytes': 69460.0,
            '_mnt_path1.xprt.tcp.port': 0.0,
            '_mnt_path1.xprt.tcp.backlogutil': 11896527.0,
            '_mnt_path1.rpc.access.ops': 2988.0,
            '_mnt_path1.rpc.write.ops': 16.0
        }

        unpublished_metrics = {
            '_mnt_path2.events.delay': 0.0
        }

        self.collector.MOUNTSTATS = self.getFixturePath('mountstats_1')
        self.collector.collect()
        self.assertPublishedMany(publish_mock, {})

        self.collector.MOUNTSTATS = self.getFixturePath('mountstats_2')
        self.collector.collect()
        self.assertPublishedMany(publish_mock, published_metrics)
        self.assertUnpublishedMany(publish_mock, unpublished_metrics)

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python
# coding=utf-8
################################################################################

from test import CollectorTestCase
from test import get_collector_config
from test import unittest

from passenger_stats import PassengerCollector

################################################################################


class TestPassengerCollector(CollectorTestCase):
    def setUp(self):
        config = get_collector_config('PassengerCollector', {})
        self.collector = PassengerCollector(config, None)

    def test_import(self):
        self.assertTrue(PassengerCollector)

################################################################################
if __name__ == "__main__":
    unittest.main()



# coding=utf-8

"""
This class collects data from plain text files

#### Dependencies

"""

import diamond.collector
import os
import re

_RE = re.compile(r'([a-z0-9._-]+)[\s=:]([0-9-]+)(\.?\d*)')


class FilesCollector(diamond.collector.Collector):

    def get_default_config_help(self):
        config_help = super(FilesCollector, self).get_default_config_help()
        config_help.update({
            'path': 'Prefix added to all stats collected by this module, a '
                    'single dot means don''t add prefix',
            'dir': 'The directory that the performance files are in',
            'delete': 'Delete files after they are picked up',
        })
        return config_help

    def get_default_config(self):
        """
        Returns default collector settings.
        """
        config = super(FilesCollector, self).get_default_config()
        config.update({
            'path': '.',
            'dir': '/tmp/diamond',
            'delete': False,
        })
        return config

    def collect(self):
        if os.path.exists(self.config['dir']):
            for fn in os.listdir(self.config['dir']):
                if os.path.isfile(os.path.join(self.config['dir'], fn)):
                    try:
                        fh = open(os.path.join(self.config['dir'], fn))
                        found = False
                        for line in fh:
                            m = _RE.match(line)
                            if (m):
                                self.publish(
                                    m.groups()[0],
                                    m.groups()[1] + m.groups()[2],
                                    precision=max(0, len(m.groups()[2]) - 1))
                                found = True
                        fh.close()
                        if (found and self.config['delete']):
                            os.unlink(os.path.join(self.config['dir'], fn))
                    except:
                        pass



#!/usr/bin/python
# coding=utf-8
###############################################################################

from test import CollectorTestCase
from test import get_collector_config
from test import unittest
from mock import Mock
from mock import patch

try:
    from cStringIO import StringIO
    StringIO  # workaround for pyflakes issue #13
except ImportError:
    from StringIO import StringIO

from diamond.collector import Collector
from vmstat import VMStatCollector

###############################################################################


class TestVMStatCollector(CollectorTestCase):
    def setUp(self):
        config = get_collector_config('VMStatCollector', {
            'interval': 10
        })

        self.collector = VMStatCollector(config, None)

    def test_import(self):
        self.assertTrue(VMStatCollector)

    @patch('__builtin__.open')
    @patch('os.access', Mock(return_value=True))
    @patch.object(Collector, 'publish')
    def test_should_open_proc_vmstat(self, publish_mock, open_mock):
        open_mock.return_value = StringIO('')
        self.collector.collect()
        open_mock.assert_called_once_with('/proc/vmstat')

    @patch.object(Collector, 'publish')
    def test_should_work_with_real_data(self, publish_mock):
        VMStatCollector.PROC = self.getFixturePath('proc_vmstat_1')
        self.collector.collect()

        self.assertPublishedMany(publish_mock, {})

        VMStatCollector.PROC = self.getFixturePath('proc_vmstat_2')
        self.collector.collect()

        metrics = {
            'pgpgin': 0.0,
            'pgpgout': 9.2,
            'pswpin': 0.0,
            'pswpout': 0.0,
        }

        self.setDocExample(collector=self.collector.__class__.__name__,
                           metrics=metrics,
                           defaultpath=self.collector.config['path'])
        self.assertPublishedMany(publish_mock, metrics)


###############################################################################
if __name__ == "__main__":
    unittest.main()



# coding=utf-8

"""
Uses lsof to collect data on number of open files per user per type

#### Config Options

 * user_include - This is list of users to collect data for. If this is left
    empty, its a wildcard to collector for all users (default = None)
 * user_exclude - This is a list of users to exclude from collecting data. If
    this is left empty, no specific users will be excluded (default = None)
 * group_include - This is a list of groups to include in data collection. If
    this DOES NOT override user_exclude. (default = None)
 * group_exclude - This is a list of groups to exclude from collecting data.
    It DOES NOT override user_include. (default = None)
 * uid_min - This creates a floor for the user's uid. This means that it WILL
    NOT collect data for any user with a uid LOWER than the specified minimum,
    unless the user is told to be included by user_include (default = None)
 * uid_max - This creates a ceiling for the user's uid. This means that it WILL
    NOT collect data for any user with a uid HIGHER than the specified maximum,
    unless the user is told to be included by user_include (default = None)

*** Priority Explaination ***
 This is an explainatino of the priority in which users, groups, and uid, are
    evaluated. EXLCUDE ALWAYS OVERRULES INCLUDE within the same level (ie within
    users or group)
  * user_include/exclude (top level/priority)
    * group_include/exclude (second level: if user not in user_include/exclude,
          groups takes affect)
      * uid_min/max (third level: if user not met above qualifications, uids
            take affect)

 * type_include - This is a list of file types to collect ('REG', 'DIR", 'FIFO'
    , etc). If left empty, will collect for all file types. (Note: it suggested
    to not leave type_include empty, as it would add significant load to your
    graphite box(es) (default = None)
 * type_exclude - This is a list of tile types to exlude from being collected
    for. If left empty, no file types will be excluded. (default = None)

 * collect_user_data - This enables or disables the collection of user specific
    file handles. (default = False)

#### Dependencies

 * /proc/sys/fs/file-nr
 * /usr/sbin/lsof

"""

import diamond.collector
import re
import os

_RE = re.compile(r'(\d+)\s+(\d+)\s+(\d+)')


class FilestatCollector(diamond.collector.Collector):

    PROC = '/proc/sys/fs/file-nr'

    def get_default_config_help(self):
        config_help = super(FilestatCollector, self).get_default_config_help()
        config_help.update({
        })
        return config_help

    def get_default_config(self):
        """
        Returns the default collector settings
        """
        config = super(FilestatCollector, self).get_default_config()
        config.update({
            'path':     'files',
            'method':   'Threaded',
            'user_include': None,
            'user_exclude': None,
            'group_include': None,
            'group_exclude': None,
            'uid_min': 0,
            'uid_max': 65536,
            'type_include': None,
            'type_exclude': None,
            'collect_user_data': False
        })
        return config

    def get_userlist(self):
        """
        This collects all the users with open files on the system, and filters
        based on the variables user_include and user_exclude
        """
    # convert user/group  lists to arrays if strings
        if isinstance(self.config['user_include'], str):
            self.config['user_include'] = self.config['user_include'].split()
        if isinstance(self.config['user_exclude'], str):
            self.config['user_exclude'] = self.config['user_exclude'].split()
        if isinstance(self.config['group_include'], str):
            self.config['group_include'] = self.config['group_include'].split()
        if isinstance(self.config['group_exclude'], str):
            self.config['group_exclude'] = self.config['group_exclude'].split()

        rawusers = os.popen("lsof | awk '{ print $3 }' | sort | uniq -d"
                            ).read().split()
        userlist = []

        # remove any not on the user include list
        if (self.config['user_include'] is None
                or len(self.config['user_include']) == 0):
            userlist = rawusers
        else:
            # only work with specified include list, which is added at the end
            userlist = []

        # add any user in the group include list
        addedByGroup = []
        if (self.config['group_include'] is not None
                and len(self.config['group_include']) > 0):
            for u in rawusers:
                self.log.info(u)
                # get list of groups of user
                user_groups = os.popen("id -Gn %s" % (u)).read().split()
                for gi in self.config['group_include']:
                    if gi in user_groups and u not in userlist:
                        userlist.append(u)
                        addedByGroup.append(u)
                        break

        # remove any user in the exclude group list
        if (self.config['group_exclude'] is not None
                and len(self.config['group_exclude']) > 0):
            # create tmp list to iterate over while editing userlist
            tmplist = userlist[:]
            for u in tmplist:
                # get list of groups of user
                groups = os.popen("id -Gn %s" % (u)).read().split()
                for gi in self.config['group_exclude']:
                    if gi in groups:
                        userlist.remove(u)
                        break

        # remove any that aren't within the uid limits
        # make sure uid_min/max are ints
        self.config['uid_min'] = int(self.config['uid_min'])
        self.config['uid_max'] = int(self.config['uid_max'])
        tmplist = userlist[:]
        for u in tmplist:
            if (self.config['user_include'] is None
                    or u not in self.config['user_include']):
                if u not in addedByGroup:
                    uid = int(os.popen("id -u %s" % (u)).read())
                    if (uid < self.config['uid_min']
                            and self.config['uid_min'] is not None
                            and u in userlist):
                        userlist.remove(u)
                    if (uid > self.config['uid_max']
                            and self.config['uid_max'] is not None
                            and u in userlist):
                        userlist.remove(u)

        # add users that are in the users include list
        if self.config['user_include'] is not None and len(
                self.config['user_include']) > 0:
            for u in self.config['user_include']:
                if u in rawusers and u not in userlist:
                    userlist.append(u)

        # remove any that is on the user exclude list
        if self.config['user_exclude'] is not None and len(
                self.config['user_exclude']) > 0:
            for u in self.config['user_exclude']:
                if u in userlist:
                    userlist.remove(u)

        return userlist

    def get_typelist(self):
        """
        This collects all avaliable types and applies include/exclude filters
        """
        typelist = []

        # convert type list into arrays if strings
        if isinstance(self.config['type_include'], str):
            self.config['type_include'] = self.config['type_include'].split()
        if isinstance(self.config['type_exclude'], str):
            self.config['type_exclude'] = self.config['type_exclude'].split()

        # remove any not in include list
        if self.config['type_include'] is None or len(
                self.config['type_include']) == 0:
            typelist = os.popen("lsof | awk '{ print $5 }' | sort | uniq -d"
                                ).read().split()
        else:
            typelist = self.config['type_include']

        # remove any in the exclude list
        if self.config['type_exclude'] is not None and len(
                self.config['type_include']) > 0:
            for t in self.config['type_exclude']:
                if t in typelist:
                    typelist.remove(t)

        return typelist

    def process_lsof(self, users, types):
        """
        Get the list of users and file types to collect for and collect the
        data from lsof
        """
        d = {}
        for u in users:
            d[u] = {}
            tmp = os.popen("lsof -bu %s | awk '{ print $5 }'" % (
                u)).read().split()
            for t in types:
                d[u][t] = tmp.count(t)
        return d

    def collect(self):
        if not os.access(self.PROC, os.R_OK):
            return None

        # collect total open files
        file = open(self.PROC)
        for line in file:
            match = _RE.match(line)
            if match:
                self.publish('assigned', int(match.group(1)))
                self.publish('unused',   int(match.group(2)))
                self.publish('max',      int(match.group(3)))
        file.close()

        # collect open files per user per type
        if self.config['collect_user_data']:
            data = self.process_lsof(self.get_userlist(), self.get_typelist())
            for ukey in data.iterkeys():
                for tkey in data[ukey].iterkeys():
                    self.log.info('files.user.%s.%s %s' % (
                        ukey, tkey, int(data[ukey][tkey])))
                    self.publish('user.%s.%s' % (ukey, tkey),
                                 int(data[ukey][tkey]))



#!/usr/bin/python
# coding=utf-8
################################################################################

from test import CollectorTestCase
from test import get_collector_config
from test import unittest
from mock import Mock
from mock import patch

try:
    from cStringIO import StringIO
    StringIO  # workaround for pyflakes issue #13
except ImportError:
    from StringIO import StringIO

from diamond.collector import Collector
from interrupt import InterruptCollector

################################################################################


class TestInterruptCollector(CollectorTestCase):
    def setUp(self):
        config = get_collector_config('InterruptCollector', {
            'interval': 1
        })

        self.collector = InterruptCollector(config, None)

    def test_import(self):
        self.assertTrue(InterruptCollector)

    @patch('__builtin__.open')
    @patch('os.access', Mock(return_value=True))
    @patch.object(Collector, 'publish')
    def test_should_open_proc_stat(self, publish_mock, open_mock):
        open_mock.return_value = StringIO('')
        self.collector.collect()
        open_mock.assert_called_once_with('/proc/interrupts', 'r')

    @patch.object(Collector, 'publish')
    def test_should_work_with_real_data_24_core(self, publish_mock):
        InterruptCollector.PROC = self.getFixturePath('interrupts_24_core_1')
        self.collector.collect()

        self.assertPublishedMany(publish_mock, {})

        InterruptCollector.PROC = self.getFixturePath('interrupts_24_core_2')
        self.collector.collect()

        metrics = {
            'IO-APIC-edge.timer.0.CPU0': 318660.000000,
            'IO-APIC-edge.timer.0.total': 318660.000000,
            'PCI-MSI-X.eth3-rx-1.51.CPU6': 293.000000,
            'PCI-MSI-X.eth3-rx-1.51.CPU7': 330.000000,
            'PCI-MSI-X.eth3-rx-1.51.CPU9': 286.000000,
            'PCI-MSI-X.eth3-rx-1.51.total': 909.000000,
            'PCI-MSI-X.eth3-rx-2.59.CPU21': 98790.000000,
            'PCI-MSI-X.eth3-rx-2.59.total': 98790.000000,
            'PCI-MSI-X.eth3-rx-3.67.CPU7': 743.000000,
            'PCI-MSI-X.eth3-rx-3.67.CPU9': 378.000000,
            'PCI-MSI-X.eth3-rx-3.67.total': 1121.000000,
            'PCI-MSI-X.eth3-tx-0.75.CPU23': 304345.000000,
            'PCI-MSI-X.eth3-tx-0.75.total': 304345.000000,
            'IO-APIC-level_3w-sas.CPU6': 301014.000000,
            'IO-APIC-level_3w-sas.total': 301014.000000,
            'PCI-MSI-X_eth2-rx-0.CPU20': 20570.000000,
            'PCI-MSI-X_eth2-rx-0.total': 20570.000000,
            'PCI-MSI-X_eth2-rx-1.CPU6': 94.000000,
            'PCI-MSI-X_eth2-rx-1.CPU7': 15.000000,
            'PCI-MSI-X_eth2-rx-1.CPU9': 50.000000,
            'PCI-MSI-X_eth2-rx-1.total': 159.000000,
            'PCI-MSI-X_eth2-rx-2.CPU17': 159.000000,
            'PCI-MSI-X_eth2-rx-2.total': 159.000000,
            'PCI-MSI-X_eth2-rx-3.CPU8': 159.000000,
            'PCI-MSI-X_eth2-rx-3.total': 159.000000,
            'PCI-MSI-X_eth2-tx-0.CPU16': 159.000000,
            'PCI-MSI-X_eth2-tx-0.total': 159.000000,
            'PCI-MSI_eth0.CPU18': 10397.000000,
            'PCI-MSI_eth0.total': 10397.000000,
            'PCI-MSI-X_eth3-rx-0.CPU22': 224074.000000,
            'PCI-MSI-X_eth3-rx-0.total': 224074.000000,
            'PCI-MSI_eth1.CPU19': 10386.000000,
            'PCI-MSI_eth1.total': 10386.000000,
        }

        self.setDocExample(collector=self.collector.__class__.__name__,
                           metrics=metrics,
                           defaultpath=self.collector.config['path'])
        self.assertPublishedMany(publish_mock, metrics)

    @patch.object(Collector, 'publish')
    def test_should_work_with_real_data_kvm(self, publish_mock):
        InterruptCollector.PROC = self.getFixturePath('interrupts_kvm_1')
        self.collector.collect()

        self.assertPublishedMany(publish_mock, {})

        InterruptCollector.PROC = self.getFixturePath('interrupts_kvm_2')
        self.collector.collect()

        self.assertPublishedMany(publish_mock, {
            'IO-APIC-edge.timer.0.CPU0': 279023.000000,
            'IO-APIC-edge.timer.0.total': 279023.000000,
            'IO-APIC-level.virtio0-virtio1.10.CPU0': 15068.000000,
            'IO-APIC-level.virtio0-virtio1.10.total': 15068.000000,
            'LOC.CPU0': 278993.000000,
            'LOC.CPU1': 279000.000000,
            'LOC.total': 557993.000000,
        })

################################################################################
if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python
# coding=utf-8
################################################################################

from test import CollectorTestCase
from test import get_collector_config
from test import unittest
from test import run_only
from mock import patch

from diamond.collector import Collector
from users import UsersCollector

import sys


################################################################################


def run_only_if_pyutmp_is_available(func):
    try:
        import pyutmp
        pyutmp  # workaround for pyflakes issue #13
    except ImportError:
        pyutmp = None
    try:
        import utmp
        utmp  # workaround for pyflakes issue #13
    except ImportError:
        utmp = None
    pred = lambda: pyutmp is not None or utmp is not None
    return run_only(func, pred)


class TestUsersCollector(CollectorTestCase):
    def setUp(self):
        config = get_collector_config('UsersCollector', {
            'utmp': self.getFixturePath('utmp.centos6'),
        })

        self.collector = UsersCollector(config, None)

    def test_import(self):
        self.assertTrue(UsersCollector)

    @run_only_if_pyutmp_is_available
    @patch.object(Collector, 'publish')
    def test_should_work_with_real_data(self, publish_mock):

        metrics = {
            'kormoc':   2,
            'root':     3,
            'total':    5,
        }

        self.setDocExample(collector=self.collector.__class__.__name__,
                           metrics=metrics,
                           defaultpath=self.collector.config['path'])

        # Because of the compiled nature of pyutmp, we can't actually test
        # different operating system versions then the currently running
        # one
        if sys.platform.startswith('linux'):
            self.collector.collect()

            self.assertPublishedMany(publish_mock, metrics)

################################################################################
if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python
# coding=utf-8
################################################################################

from test import CollectorTestCase
from test import get_collector_config
from test import unittest
from mock import Mock
from mock import patch

from diamond.collector import Collector
from bind import BindCollector

################################################################################


class TestBindCollector(CollectorTestCase):
    def setUp(self):
        config = get_collector_config('BindCollector', {
            'interval': 10,
        })

        self.collector = BindCollector(config, None)

    def test_import(self):
        self.assertTrue(BindCollector)

    @patch.object(Collector, 'publish')
    def test_should_work_with_real_data(self, publish_mock):
        patch_urlopen = patch('urllib2.urlopen', Mock(
            return_value=self.getFixture('bind.xml')))

        patch_urlopen.start()
        self.collector.collect()
        patch_urlopen.stop()

        metrics = {
            'view._default.resstat.Queryv4': 0.000000,
            'view._default.resstat.Queryv6': 0.000000,
            'view._default.resstat.Responsev4': 0.000000,
            'view._default.resstat.Responsev6': 0.000000,
            'view._default.resstat.NXDOMAIN': 0.000000,
            'view._default.resstat.SERVFAIL': 0.000000,
            'view._default.resstat.FORMERR': 0.000000,
            'view._default.resstat.OtherError': 0.000000,
            'view._default.resstat.EDNS0Fail': 0.000000,
            'view._default.resstat.Mismatch': 0.000000,
            'view._default.resstat.Truncated': 0.000000,
            'view._default.resstat.Lame': 0.000000,
            'view._default.resstat.Retry': 0.000000,
            'view._default.resstat.QueryAbort': 0.000000,
            'view._default.resstat.QuerySockFail': 0.000000,
            'view._default.resstat.QueryTimeout': 0.000000,
            'view._default.resstat.GlueFetchv4': 0.000000,
            'view._default.resstat.GlueFetchv6': 0.000000,
            'view._default.resstat.GlueFetchv4Fail': 0.000000,
            'view._default.resstat.GlueFetchv6Fail': 0.000000,
            'view._default.resstat.ValAttempt': 0.000000,
            'view._default.resstat.ValOk': 0.000000,
            'view._default.resstat.ValNegOk': 0.000000,
            'view._default.resstat.ValFail': 0.000000,
            'view._default.resstat.QryRTT10': 0.000000,
            'view._default.resstat.QryRTT100': 0.000000,
            'view._default.resstat.QryRTT500': 0.000000,
            'view._default.resstat.QryRTT800': 0.000000,
            'view._default.resstat.QryRTT1600': 0.000000,
            'view._default.resstat.QryRTT1600+': 0.000000,
            'requests.QUERY': 0.000000,
            'queries.A': 0.000000,
            'nsstat.Requestv4': 0.000000,
            'nsstat.Requestv6': 0.000000,
            'nsstat.ReqEdns0': 0.000000,
            'nsstat.ReqBadEDNSVer': 0.000000,
            'nsstat.ReqTSIG': 0.000000,
            'nsstat.ReqSIG0': 0.000000,
            'nsstat.ReqBadSIG': 0.000000,
            'nsstat.ReqTCP': 0.000000,
            'nsstat.AuthQryRej': 0.000000,
            'nsstat.RecQryRej': 0.000000,
            'nsstat.XfrRej': 0.000000,
            'nsstat.UpdateRej': 0.000000,
            'nsstat.Response': 0.000000,
            'nsstat.TruncatedResp': 0.000000,
            'nsstat.RespEDNS0': 0.000000,
            'nsstat.RespTSIG': 0.000000,
            'nsstat.RespSIG0': 0.000000,
            'nsstat.QrySuccess': 0.000000,
            'nsstat.QryAuthAns': 0.000000,
            'nsstat.QryNoauthAns': 0.000000,
            'nsstat.QryReferral': 0.000000,
            'nsstat.QryNxrrset': 0.000000,
            'nsstat.QrySERVFAIL': 0.000000,
            'nsstat.QryFORMERR': 0.000000,
            'nsstat.QryNXDOMAIN': 0.000000,
            'nsstat.QryRecursion': 0.000000,
            'nsstat.QryDuplicate': 0.000000,
            'nsstat.QryDropped': 0.000000,
            'nsstat.QryFailure': 0.000000,
            'nsstat.XfrReqDone': 0.000000,
            'nsstat.UpdateReqFwd': 0.000000,
            'nsstat.UpdateRespFwd': 0.000000,
            'nsstat.UpdateFwdFail': 0.000000,
            'nsstat.UpdateDone': 0.000000,
            'nsstat.UpdateFail': 0.000000,
            'nsstat.UpdateBadPrereq': 0.000000,
            'zonestat.NotifyOutv4': 0.000000,
            'zonestat.NotifyOutv6': 0.000000,
            'zonestat.NotifyInv4': 0.000000,
            'zonestat.NotifyInv6': 0.000000,
            'zonestat.NotifyRej': 0.000000,
            'zonestat.SOAOutv4': 0.000000,
            'zonestat.SOAOutv6': 0.000000,
            'zonestat.AXFRReqv4': 0.000000,
            'zonestat.AXFRReqv6': 0.000000,
            'zonestat.IXFRReqv4': 0.000000,
            'zonestat.IXFRReqv6': 0.000000,
            'zonestat.XfrSuccess': 0.000000,
            'zonestat.XfrFail': 0.000000,
            'sockstat.UDP4Open': 0.000000,
            'sockstat.UDP6Open': 0.000000,
            'sockstat.TCP4Open': 0.000000,
            'sockstat.TCP6Open': 0.000000,
            'sockstat.UnixOpen': 0.000000,
            'sockstat.UDP4OpenFail': 0.000000,
            'sockstat.UDP6OpenFail': 0.000000,
            'sockstat.TCP4OpenFail': 0.000000,
            'sockstat.TCP6OpenFail': 0.000000,
            'sockstat.UnixOpenFail': 0.000000,
            'sockstat.UDP4Close': 0.000000,
            'sockstat.UDP6Close': 0.000000,
            'sockstat.TCP4Close': 0.000000,
            'sockstat.TCP6Close': 0.000000,
            'sockstat.UnixClose': 0.000000,
            'sockstat.FDWatchClose': 0.000000,
            'sockstat.UDP4BindFail': 0.000000,
            'sockstat.UDP6BindFail': 0.000000,
            'sockstat.TCP4BindFail': 0.000000,
            'sockstat.TCP6BindFail': 0.000000,
            'sockstat.UnixBindFail': 0.000000,
            'sockstat.FdwatchBindFail': 0.000000,
            'sockstat.UDP4ConnFail': 0.000000,
            'sockstat.UDP6ConnFail': 0.000000,
            'sockstat.TCP4ConnFail': 0.000000,
            'sockstat.TCP6ConnFail': 0.000000,
            'sockstat.UnixConnFail': 0.000000,
            'sockstat.FDwatchConnFail': 0.000000,
            'sockstat.UDP4Conn': 0.000000,
            'sockstat.UDP6Conn': 0.000000,
            'sockstat.TCP4Conn': 0.000000,
            'sockstat.TCP6Conn': 0.000000,
            'sockstat.UnixConn': 0.000000,
            'sockstat.FDwatchConn': 0.000000,
            'sockstat.TCP4AcceptFail': 0.000000,
            'sockstat.TCP6AcceptFail': 0.000000,
            'sockstat.UnixAcceptFail': 0.000000,
            'sockstat.TCP4Accept': 0.000000,
            'sockstat.TCP6Accept': 0.000000,
            'sockstat.UnixAccept': 0.000000,
            'sockstat.UDP4SendErr': 0.000000,
            'sockstat.UDP6SendErr': 0.000000,
            'sockstat.TCP4SendErr': 0.000000,
            'sockstat.TCP6SendErr': 0.000000,
            'sockstat.UnixSendErr': 0.000000,
            'sockstat.FDwatchSendErr': 0.000000,
            'sockstat.UDP4RecvErr': 0.000000,
            'sockstat.UDP6RecvErr': 0.000000,
            'sockstat.TCP4RecvErr': 0.000000,
            'sockstat.TCP6RecvErr': 0.000000,
            'sockstat.UnixRecvErr': 0.000000,
            'sockstat.FDwatchRecvErr': 0.000000,
        }

        self.setDocExample(collector=self.collector.__class__.__name__,
                           metrics=metrics,
                           defaultpath=self.collector.config['path'])
        self.assertPublishedMany(publish_mock, metrics)


################################################################################
if __name__ == "__main__":
    unittest.main()



# coding=utf-8

"""
Shells out to get ipvs statistics, which may or may not require sudo access

#### Dependencies

 * /usr/sbin/ipvsadmin

"""

import diamond.collector
import subprocess
import os
import string
from diamond.collector import str_to_bool


class IPVSCollector(diamond.collector.Collector):

    def __init__(self, config, handlers):
        super(IPVSCollector, self).__init__(config, handlers)

        # Verify the --exact flag works
        self.statcommand = [self.config['bin'], '--list', '--stats',
                            '--numeric', '--exact']
        self.concommand = [self.config['bin'], '--list', '--numeric']

        if str_to_bool(self.config['use_sudo']):
            self.statcommand.insert(0, self.config['sudo_cmd'])
            self.concommand.insert(0, self.config['sudo_cmd'])
            # The -n (non-interactive) option prevents sudo from
            # prompting the user for a password.
            self.statcommand.insert(1, '-n')
            self.concommand.insert(1, '-n')

        p = subprocess.Popen(self.statcommand, stdout=subprocess.PIPE,
                             stderr=subprocess.PIPE)
        p.wait()

        if p.returncode == 255:
            self.statcommand = filter(
                lambda a: a != '--exact', self.statcommand)

    def get_default_config_help(self):
        config_help = super(IPVSCollector, self).get_default_config_help()
        config_help.update({
            'bin': 'Path to ipvsadm binary',
            'use_sudo': 'Use sudo?',
            'sudo_cmd': 'Path to sudo',
        })
        return config_help

    def get_default_config(self):
        """
        Returns the default collector settings
        """
        config = super(IPVSCollector, self).get_default_config()
        config.update({
            'bin':              '/usr/sbin/ipvsadm',
            'use_sudo':         True,
            'sudo_cmd':         '/usr/bin/sudo',
            'path':             'ipvs'
        })
        return config

    def collect(self):
        if not os.access(self.config['bin'], os.X_OK):
            self.log.error("%s is not executable", self.config['bin'])
            return False

        if (str_to_bool(self.config['use_sudo'])
                and not os.access(self.config['sudo_cmd'], os.X_OK)):
            self.log.error("%s is not executable", self.config['sudo_cmd'])
            return False

        p = subprocess.Popen(self.statcommand,
                             stdout=subprocess.PIPE).communicate()[0][:-1]

        columns = {
            'conns': 2,
            'inpkts': 3,
            'outpkts': 4,
            'inbytes': 5,
            'outbytes': 6,
        }

        external = ""
        backend = ""
        for i, line in enumerate(p.split("\n")):
            if i < 3:
                continue
            row = line.split()

            if row[0] == "TCP" or row[0] == "UDP":
                external = row[0] + "_" + string.replace(row[1], ".", "_")
                backend = "total"
            elif row[0] == "->":
                backend = string.replace(row[1], ".", "_")
            else:
                continue

            for metric, column in columns.iteritems():
                metric_name = ".".join([external, backend, metric])
                # metric_value = int(row[column])
                value = row[column]
                if (value.endswith('K')):
                        metric_value = int(value[0:len(value) - 1]) * 1024
                elif (value.endswith('M')):
                        metric_value = (int(value[0:len(value) - 1]) * 1024
                                        * 1024)
                elif (value.endswith('G')):
                        metric_value = (int(value[0:len(value) - 1]) * 1024.0
                                        * 1024.0 * 1024.0)
                else:
                        metric_value = float(value)

                self.publish(metric_name, metric_value)

        p = subprocess.Popen(self.concommand,
                             stdout=subprocess.PIPE).communicate()[0][:-1]

        columns = {
            'active': 4,
            'inactive': 5,
        }

        external = ""
        backend = ""
        total = {}
        for i, line in enumerate(p.split("\n")):
            if i < 3:
                continue
            row = line.split()

            if row[0] == "TCP" or row[0] == "UDP":
                if total:
                    for metric, value in total.iteritems():
                        self.publish(
                            ".".join([external, "total", metric]), value)

                for k in columns.keys():
                    total[k] = 0.0

                external = row[0] + "_" + string.replace(row[1], ".", "_")
                continue
            elif row[0] == "->":
                backend = string.replace(row[1], ".", "_")
            else:
                continue

            for metric, column in columns.iteritems():
                metric_name = ".".join([external, backend, metric])
                # metric_value = int(row[column])
                value = row[column]
                if (value.endswith('K')):
                        metric_value = int(value[0:len(value) - 1]) * 1024
                elif (value.endswith('M')):
                        metric_value = (int(value[0:len(value) - 1]) * 1024
                                        * 1024)
                elif (value.endswith('G')):
                        metric_value = (int(value[0:len(value) - 1]) * 1024.0
                                        * 1024.0 * 1024.0)
                else:
                        metric_value = float(value)

                total[metric] += metric_value
                self.publish(metric_name, metric_value)

        if total:
            for metric, value in total.iteritems():
                self.publish(".".join([external, "total", metric]), value)



#!/usr/bin/python
# coding=utf-8
################################################################################

from test import CollectorTestCase
from test import get_collector_config
from test import unittest
from test import run_only
from mock import MagicMock
from mock import patch
from mock import call

from diamond.collector import Collector
from mongodb import MongoDBCollector

################################################################################


def run_only_if_pymongo_is_available(func):
    try:
        import pymongo
        pymongo  # workaround for pyflakes issue #13
    except ImportError:
        pymongo = None
    pred = lambda: pymongo is not None
    return run_only(func, pred)


class TestMongoDBCollector(CollectorTestCase):
    def setUp(self):
        config = get_collector_config('MongoDBCollector', {
            'host': 'localhost:27017',
            'databases': '^db',
        })
        self.collector = MongoDBCollector(config, None)
        self.connection = MagicMock()

    def test_import(self):
        self.assertTrue(MongoDBCollector)

    @run_only_if_pymongo_is_available
    @patch('pymongo.Connection')
    @patch.object(Collector, 'publish')
    def test_should_publish_nested_keys_for_server_stats(self,
                                                         publish_mock,
                                                         connector_mock):
        data = {'more_keys': {'nested_key': 1}, 'key': 2, 'string': 'str'}
        self._annotate_connection(connector_mock, data)

        self.collector.collect()

        self.connection.db.command.assert_called_once_with('serverStatus')
        self.assertPublishedMany(publish_mock, {
            'more_keys.nested_key': 1,
            'key': 2
        })

    @run_only_if_pymongo_is_available
    @patch('pymongo.Connection')
    @patch.object(Collector, 'publish')
    def test_should_publish_nested_keys_for_db_stats(self,
                                                     publish_mock,
                                                     connector_mock):
        data = {'db_keys': {'db_nested_key': 1}, 'dbkey': 2, 'dbstring': 'str'}
        self._annotate_connection(connector_mock, data)

        self.collector.collect()

        self.connection['db1'].command.assert_called_once_with('dbStats')
        metrics = {
            'db_keys.db_nested_key': 1,
            'dbkey': 2
        }

        self.setDocExample(collector=self.collector.__class__.__name__,
                           metrics=metrics,
                           defaultpath=self.collector.config['path'])
        self.assertPublishedMany(publish_mock, metrics)

    @run_only_if_pymongo_is_available
    @patch('pymongo.Connection')
    @patch.object(Collector, 'publish')
    def test_should_publish_stats_with_long_type(self,
                                                 publish_mock,
                                                 connector_mock):
        data = {'more_keys': long(1), 'key': 2, 'string': 'str'}
        self._annotate_connection(connector_mock, data)

        self.collector.collect()

        self.connection.db.command.assert_called_once_with('serverStatus')
        self.assertPublishedMany(publish_mock, {
            'more_keys': 1,
            'key': 2
        })

    @run_only_if_pymongo_is_available
    @patch('pymongo.Connection')
    @patch.object(Collector, 'publish')
    def test_should_ignore_unneeded_databases(self,
                                              publish_mock,
                                              connector_mock):
        self._annotate_connection(connector_mock, {})

        self.collector.collect()

        assert call('baddb') not in self.connection.__getitem__.call_args_list

    @run_only_if_pymongo_is_available
    @patch('pymongo.Connection')
    @patch.object(Collector, 'publish')
    def test_should_ignore_unneeded_collections(self,
                                                publish_mock,
                                                connector_mock):
        data = {'more_keys': long(1), 'key': 2, 'string': 'str'}
        self._annotate_connection(connector_mock, data)

        self.connection['db1'].collection_names.return_value = ['collection1',
                                                                'tmp.mr.tmp1']
        self.connection['db1'].command.return_value = {'key': 2,
                                                       'string': 'str'}

        self.collector.collect()

        self.connection.db.command.assert_called_once_with('serverStatus')
        self.connection['db1'].collection_names.assert_called_once_with()
        self.connection['db1'].command.assert_any_call('dbStats')
        self.connection['db1'].command.assert_any_call('collstats',
                                                       'collection1')
        assert call('collstats', 'tmp.mr.tmp1') not in self.connection['db1'].command.call_args_list  # NOQA
        metrics = {
            'databases.db1.collection1.key': 2,
        }

        self.assertPublishedMany(publish_mock, metrics)

    def _annotate_connection(self, connector_mock, data):
        connector_mock.return_value = self.connection
        self.connection.db.command.return_value = data
        self.connection.database_names.return_value = ['db1', 'baddb']


class TestMongoMultiHostDBCollector(CollectorTestCase):
    def setUp(self):
        config = get_collector_config('MongoDBCollector', {
            'hosts': ['localhost:27017', 'localhost:27057'],
            'databases': '^db',
        })
        self.collector = MongoDBCollector(config, None)
        self.connection = MagicMock()

    def test_import(self):
        self.assertTrue(MongoDBCollector)

    @run_only_if_pymongo_is_available
    @patch('pymongo.Connection')
    @patch.object(Collector, 'publish')
    def test_should_publish_nested_keys_for_server_stats(self,
                                                         publish_mock,
                                                         connector_mock):
        data = {'more_keys': {'nested_key': 1}, 'key': 2, 'string': 'str'}
        self._annotate_connection(connector_mock, data)

        self.collector.collect()

        self.connection.db.command.assert_called_with('serverStatus')
        self.assertPublishedMany(publish_mock, {
            'localhost_27017.more_keys.nested_key': 1,
            'localhost_27057.more_keys.nested_key': 1,
            'localhost_27017.key': 2,
            'localhost_27057.key': 2
        })

    @run_only_if_pymongo_is_available
    @patch('pymongo.Connection')
    @patch.object(Collector, 'publish')
    def test_should_publish_nested_keys_for_db_stats(self,
                                                     publish_mock,
                                                     connector_mock):
        data = {'db_keys': {'db_nested_key': 1}, 'dbkey': 2, 'dbstring': 'str'}
        self._annotate_connection(connector_mock, data)

        self.collector.collect()

        self.connection['db1'].command.assert_called_with('dbStats')
        metrics = {
            'localhost_27017.db_keys.db_nested_key': 1,
            'localhost_27057.db_keys.db_nested_key': 1,
            'localhost_27017.dbkey': 2,
            'localhost_27057.dbkey': 2
        }

        self.setDocExample(collector=self.collector.__class__.__name__,
                           metrics=metrics,
                           defaultpath=self.collector.config['path'])
        self.assertPublishedMany(publish_mock, metrics)

    @run_only_if_pymongo_is_available
    @patch('pymongo.Connection')
    @patch.object(Collector, 'publish')
    def test_should_publish_stats_with_long_type(self,
                                                 publish_mock,
                                                 connector_mock):
        data = {'more_keys': long(1), 'key': 2, 'string': 'str'}
        self._annotate_connection(connector_mock, data)

        self.collector.collect()

        self.connection.db.command.assert_called_with('serverStatus')
        self.assertPublishedMany(publish_mock, {
            'localhost_27017.more_keys': 1,
            'localhost_27057.more_keys': 1,
            'localhost_27017.key': 2,
            'localhost_27057.key': 2
        })

    @run_only_if_pymongo_is_available
    @patch('pymongo.Connection')
    @patch.object(Collector, 'publish')
    def test_should_ignore_unneeded_databases(self,
                                              publish_mock,
                                              connector_mock):
        self._annotate_connection(connector_mock, {})

        self.collector.collect()

        assert call('baddb') not in self.connection.__getitem__.call_args_list

    @run_only_if_pymongo_is_available
    @patch('pymongo.Connection')
    @patch.object(Collector, 'publish')
    def test_should_ignore_unneeded_collections(self,
                                                publish_mock,
                                                connector_mock):
        data = {'more_keys': long(1), 'key': 2, 'string': 'str'}
        self._annotate_connection(connector_mock, data)

        self.connection['db1'].collection_names.return_value = ['collection1',
                                                                'tmp.mr.tmp1']
        self.connection['db1'].command.return_value = {'key': 2,
                                                       'string': 'str'}

        self.collector.collect()

        self.connection.db.command.assert_called_with('serverStatus')
        self.connection['db1'].collection_names.assert_called_with()
        self.connection['db1'].command.assert_any_call('dbStats')
        self.connection['db1'].command.assert_any_call('collstats',
                                                       'collection1')
        assert call('collstats', 'tmp.mr.tmp1') not in self.connection['db1'].command.call_args_list  # NOQA
        metrics = {
            'localhost_27017.databases.db1.collection1.key': 2,
            'localhost_27057.databases.db1.collection1.key': 2,
        }

        self.assertPublishedMany(publish_mock, metrics)

    def _annotate_connection(self, connector_mock, data):
        connector_mock.return_value = self.connection
        self.connection.db.command.return_value = data
        self.connection.database_names.return_value = ['db1', 'baddb']


################################################################################
if __name__ == "__main__":
    unittest.main()



# coding=utf-8

"""
Diamond handler that check if values are too high or too low, if so send an
alert to a Sentry server

This handler requires the Python module Raven:
http://raven.readthedocs.org/en/latest/index.html

To work this handler need a similar configuration:

[[SentryHandler]]

# Create a new project in Sentry and copy the DSN here:
dsn = http://user:pass@hostname/id

[[[load]]]

name = Load Average
# check for load average of the last 15 minutes
path = loadavg.15
max = 8.5

[[[free_memory]]]

name = Free Memory
path = memory.MemFree
min = 66020000
"""

__author__ = 'Bruno Clermont'
__email__ = 'bruno.clermont@gmail.com'

import logging
import re

from Handler import Handler
from diamond.collector import get_hostname
from configobj import Section

try:
    import raven.handlers.logging
except ImportError:
    raven = None


class InvalidRule(ValueError):
    """
    invalid rule
    """
    pass


class BaseResult(object):
    """
    Base class for a Rule minimum/maximum check result
    """
    adjective = None

    def __init__(self, value, threshold):
        """
        @type value: float
        @param value: metric value
        @type threshold: float
        @param threshold: value that trigger a warning
        """
        self.value = value
        self.threshold = threshold

        if not raven:
            self.log.error('raven.handlers.logging import failed. '
                           'Handler disabled')

    @property
    def verbose_message(self):
        """return more complete message"""
        if self.threshold is None:
            return 'No threshold'
        return '%.1f is %s than %.1f' % (self.value,
                                         self.adjective,
                                         self.threshold)

    @property
    def _is_error(self):
        raise NotImplementedError('_is_error')

    @property
    def is_error(self):
        """
        for some reason python do this:
        >>> 1.0 > None
        True
        >>> 1.0 < None
        False
        so we just check if min/max is not None before return _is_error
        """
        if self.threshold is None:
            return False
        return self._is_error

    def __str__(self):
        name = self.__class__.__name__.lower()
        if self.threshold is None:
            return '%s: %.1f no threshold' % (name, self.value)
        return '%.1f (%s: %.1f)' % (self.value, name, self.threshold)


class Minimum(BaseResult):
    """
    Minimum result
    """
    adjective = 'lower'

    @property
    def _is_error(self):
        """if it's too low"""
        return self.value < self.threshold


class Maximum(BaseResult):
    """
    Maximum result
    """
    adjective = 'higher'

    @property
    def _is_error(self):
        """if it's too high"""
        return self.value > self.threshold


class Rule(object):
    """
    Alert rule
    """

    def __init__(self, name, path, min=None, max=None):
        """
        @type name: string
        @param name: rule name, used to identify this rule in Sentry
        @type path: string
        @param path: un-compiled regular expression of the path of the rule
        @type min: string of float/int, int or float. will be convert to float
        @param min: optional minimal value that if value goes below it send
            an alert to Sentry
        @type max: string of float/int, int or float. will be convert to float
        @param max: optional maximal value that if value goes over it send
            an alert to Sentry
        """
        self.name = name
        # counters that can be used to debug rule
        self.counter_errors = 0
        self.counter_pass = 0

        # force min and max to be float
        try:
            self.min = float(min)
        except TypeError:
            self.min = None
        try:
            self.max = float(max)
        except TypeError:
            self.max = None

        if self.min is None and self.max is None:
            raise InvalidRule("%s: %s: both min and max are unset or invalid"
                              % (name, path))

        if self.min is not None and self.max is not None:
            if self.min > self.max:
                raise InvalidRule("min %.1f is larger than max %.1f" % (
                    self.min, self.max))

        # compile path regular expression
        self.regexp = re.compile(r'(?P<prefix>.*)\.(?P<path>%s)$' % path)

    def process(self, metric, handler):
        """
        process a single diamond metric
        @type metric: diamond.metric.Metric
        @param metric: metric to process
        @type handler: diamond.handler.sentry.SentryHandler
        @param handler: configured Sentry graphite handler
        @rtype None
        """
        match = self.regexp.match(metric.path)
        if match:
            minimum = Minimum(metric.value, self.min)
            maximum = Maximum(metric.value, self.max)

            if minimum.is_error or maximum.is_error:
                self.counter_errors += 1
                message = "%s Warning on %s: %.1f" % (self.name,
                                                      handler.hostname,
                                                      metric.value)
                culprit = "%s %s" % (handler.hostname, match.group('path'))
                handler.raven_logger.error(message, extra={
                    'culprit': culprit,
                    'data': {
                        'metric prefix': match.group('prefix'),
                        'metric path': match.group('path'),
                        'minimum check': minimum.verbose_message,
                        'maximum check': maximum.verbose_message,
                        'metric original path': metric.path,
                        'metric value': metric.value,
                        'metric precision': metric.precision,
                        'metric timestamp': metric.timestamp,
                        'minimum threshold': self.min,
                        'maximum threshold': self.max,
                        'path regular expression': self.regexp.pattern,
                        'total errors': self.counter_errors,
                        'total pass': self.counter_pass,
                        'hostname': handler.hostname
                    }
                }
                )
            else:
                self.counter_pass += 1

    def __repr__(self):
        return '%s: min:%s max:%s %s' % (self.name, self.min, self.max,
                                         self.regexp.pattern)


class SentryHandler(Handler):
    """
    Diamond handler that check if a metric goes too low or too high
    """

    # valid key name in rules sub-section
    VALID_RULES_KEYS = ('name', 'path', 'min', 'max')

    def __init__(self, config=None):
        """
        @type config: configobj.ConfigObj
        """
        Handler.__init__(self, config)
        if not raven:
            return
        # init sentry/raven
        self.sentry_log_handler = raven.handlers.logging.SentryHandler(
            self.config['dsn'])
        self.raven_logger = logging.getLogger(self.__class__.__name__)
        self.raven_logger.addHandler(self.sentry_log_handler)
        self.configure_sentry_errors()
        self.rules = self.compile_rules()
        self.hostname = get_hostname(self.config)
        if not len(self.rules):
            self.log.warning("No rules, this graphite handler is unused")

    def get_default_config_help(self):
        """
        Returns the help text for the configuration options for this handler
        """
        config = super(SentryHandler, self).get_default_config_help()

        config.update({
            'dsn': '',
        })

        return config

    def get_default_config(self):
        """
        Return the default config for the handler
        """
        config = super(SentryHandler, self).get_default_config()

        config.update({
            'dsn': '',
        })

        return config

    def compile_rules(self):
        """
        Compile alert rules
        @rtype list of Rules
        """
        output = []
        # validate configuration, skip invalid section
        for key_name, section in self.config.items():
            rule = self.compile_section(section)
            if rule is not None:
                output.append(rule)
        return output

    def compile_section(self, section):
        """
        Validate if a section is a valid rule
        @type section: configobj.Section
        @param section: section to validate
        @rtype Rule or None
        @return None if invalid
        """
        if section.__class__ != Section:
            # not a section, just skip
            return

        # name and path are mandatory
        keys = section.keys()
        for key in ('name', 'path'):
            if key not in keys:
                self.log.warning("section %s miss key '%s' ignore", key,
                                 section.name)
                return

        # just warn if invalid key in section
        for key in keys:
            if key not in self.VALID_RULES_KEYS:
                self.log.warning("invalid key %s in section %s",
                                 key, section.name)

        # need at least a min or a max
        if 'min' not in keys and 'max' not in keys:
            self.log.warning("either 'min' or 'max' is defined in %s",
                             section.name)
            return

        # add rule to the list
        kwargs = {
            'name': section['name'],
            'path': section['path']
        }
        for argument in ('min', 'max'):
            try:
                kwargs[argument] = section[argument]
            except KeyError:
                pass

        # init rule
        try:
            return Rule(**kwargs)
        except InvalidRule, err:
            self.log.error(str(err))

    def configure_sentry_errors(self):
        """
        Configure sentry.errors to use the same loggers as the root handler
        @rtype: None
        """
        sentry_errors_logger = logging.getLogger('sentry.errors')
        root_logger = logging.getLogger()
        for handler in root_logger.handlers:
            sentry_errors_logger.addHandler(handler)

    def process(self, metric):
        """
        process a single metric
        @type metric: diamond.metric.Metric
        @param metric: metric to process
        @rtype None
        """
        for rule in self.rules:
            rule.process(metric, self)

    def __repr__(self):
        return "SentryHandler '%s' %d rules" % (
            self.sentry_log_handler.client.servers, len(self.rules))



#!/usr/bin/python
# coding=utf-8
################################################################################

from test import CollectorTestCase
from test import get_collector_config
from test import unittest
from mock import Mock
from mock import patch

try:
    from cStringIO import StringIO
    StringIO  # workaround for pyflakes issue #13
except ImportError:
    from StringIO import StringIO

from diamond.collector import Collector
from filestat import FilestatCollector

################################################################################


class TestFilestatCollector(CollectorTestCase):
    def setUp(self):
        config = get_collector_config('FilestatCollector', {
            'interval': 10
        })

        self.collector = FilestatCollector(config, None)

    def test_import(self):
        self.assertTrue(FilestatCollector)

    @patch('__builtin__.open')
    @patch('os.access', Mock(return_value=True))
    @patch.object(Collector, 'publish')
    def test_should_open_proc_sys_fs_file_nr(self, publish_mock, open_mock):
        open_mock.return_value = StringIO('')
        self.collector.collect()
        open_mock.assert_called_once_with('/proc/sys/fs/file-nr')

    @patch.object(Collector, 'publish')
    def test_should_work_with_real_data(self, publish_mock):
        FilestatCollector.PROC = self.getFixturePath('proc_sys_fs_file-nr')
        self.collector.collect()

        metrics = {
            'assigned': 576,
            'unused': 0,
            'max': 4835852
        }

        self.setDocExample(collector=self.collector.__class__.__name__,
                           metrics=metrics,
                           defaultpath=self.collector.config['path'])
        self.assertPublishedMany(publish_mock, metrics)

################################################################################
if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python
# coding=utf-8
################################################################################

from test import CollectorTestCase
from test import get_collector_config
from test import unittest
from mock import Mock
from mock import patch

from diamond.collector import Collector

from monit import MonitCollector

################################################################################


class TestMonitCollector(CollectorTestCase):
    def setUp(self):
        config = get_collector_config('MonitCollector',
                                      {'byte_unit': 'kilobyte', })

        self.collector = MonitCollector(config, None)

    def test_import(self):
        self.assertTrue(MonitCollector)

    @patch.object(Collector, 'publish')
    def test_should_work_with_real_data(self, publish_mock):
        patch_urlopen = patch('urllib2.urlopen', Mock(
            return_value=self.getFixture('status.xml')))

        patch_urlopen.start()
        self.collector.collect()
        patch_urlopen.stop()

        metrics = {
            'app_thin_8101.cpu.percent': 0.9,
            'app_thin_8101.memory.kilobyte_usage': 216104,
            'app_thin_8102.cpu.percent': 1.1,
            'app_thin_8102.memory.kilobyte_usage': 212736,
            'app_thin_8103.cpu.percent': 0.9,
            'app_thin_8103.memory.kilobyte_usage': 204948,
            'app_thin_8104.cpu.percent': 0.9,
            'app_thin_8104.memory.kilobyte_usage': 212464,
            'sshd.cpu.percent': 0.0,
            'sshd.memory.kilobyte_usage': 2588,
            'rsyslogd.cpu.percent': 0.0,
            'rsyslogd.memory.kilobyte_usage': 2664,
            'postfix.cpu.percent': 0.0,
            'postfix.memory.kilobyte_usage': 2304,
            'nginx.cpu.percent': 0.0,
            'nginx.memory.kilobyte_usage': 18684,
            'haproxy.cpu.percent': 0.0,
            'haproxy.memory.kilobyte_usage': 4040,
            'cron.cpu.percent': 0.0,
            'cron.memory.kilobyte_usage': 1036,
        }

        self.setDocExample(collector=self.collector.__class__.__name__,
                           metrics=metrics,
                           defaultpath=self.collector.config['path'])
        self.assertPublishedMany(publish_mock, metrics)

    @patch.object(Collector, 'publish')
    def test_should_fail_gracefully(self, publish_mock):
        patch_urlopen = patch(
            'urllib2.urlopen',
            Mock(
                return_value=self.getFixture(
                    'status_blank.xml')))

        patch_urlopen.start()
        self.collector.collect()
        patch_urlopen.stop()

        self.assertPublishedMany(publish_mock, {})

################################################################################
if __name__ == "__main__":
    unittest.main()



# coding=utf-8

"""
VarnishCollector grabs stats from Varnish and submits them the Graphite

#### Dependencies

 * /usr/bin/varnishstat

"""

import diamond.collector
import re
import subprocess
from diamond.collector import str_to_bool


class VarnishCollector(diamond.collector.Collector):

    _RE = re.compile("^(?P<stat>[\w_\(\)\.,]*)\s+(?P<psa>\d*)\s+"
                     "(?P<psan>[\d.]*)\s(?P<desc>[\w\., /]*)$", re.M)
    _KEYS = ['client_conn', 'client_drop', 'client_req', 'cache_hit',
             'cache_hitpass', 'cache_miss', 'backend_conn', 'backend_unhealthy',
             'backend_busy', 'backend_fail', 'backend_reuse', 'backend_toolate',
             'backend_recycle', 'backend_retry', 'fetch_head', 'fetch_length',
             'fetch_chunked', 'fetch_eof', 'fetch_bad', 'fetch_close',
             'fetch_oldhttp', 'fetch_zero', 'fetch_failed', 'n_sess_mem',
             'n_sess', 'n_object', 'n_vampireobject', 'n_objectcore',
             'n_objecthead', 'n_waitinglist', 'n_vbc', 'n_wrk', 'n_wrk_create',
             'n_wrk_failed', 'n_wrk_max', 'n_wrk_lqueue', 'n_wrk_queued',
             'n_wrk_drop', 'n_backend', 'n_expired', 'n_lru_nuked',
             'n_lru_moved', 'losthdr', 'n_objsendfile', 'n_objwrite',
             'n_objoverflow', 's_sess', 's_req', 's_pipe', 's_pass', 's_fetch',
             's_hdrbytes', 's_bodybytes', 'sess_closed', 'sess_pipeline',
             'sess_readahead', 'sess_linger', 'sess_herd', 'shm_records',
             'shm_writes', 'shm_flushes', 'shm_cont', 'shm_cycles', 'sms_nreq',
             'sms_nobj', 'sms_nbytes', 'sms_balloc', 'sms_bfree', 'backend_req',
             'n_vcl', 'n_vcl_avail', 'n_vcl_discard', 'n_ban', 'n_ban_add',
             'n_ban_retire', 'n_ban_obj_test', 'n_ban_re_test', 'n_ban_dups',
             'hcb_nolock', 'hcb_lock', 'hcb_insert', 'accept_fail',
             'client_drop_late', 'uptime', 'dir_dns_lookups', 'dir_dns_failed',
             'dir_dns_hit', 'dir_dns_cache_full']

    def get_default_config_help(self):
        config_help = super(VarnishCollector, self).get_default_config_help()
        config_help.update({
            'bin':         'The path to the smartctl binary',
            'use_sudo':    'Use sudo?',
            'sudo_cmd':    'Path to sudo',
        })
        return config_help

    def get_default_config(self):
        """
        Returns the default collector settings
        """
        config = super(VarnishCollector, self).get_default_config()
        config.update({
            'path':             'varnish',
            'bin':             '/usr/bin/varnishstat',
            'use_sudo':         False,
            'sudo_cmd':         '/usr/bin/sudo',
        })
        return config

    def collect(self):
        data = {}
        output = self.poll()

        matches = self._RE.findall(output)
        for line in matches:
            if line[0] in self._KEYS:
                data[line[0]] = line[1]

        for key in data:
            self.publish(key, int(data[key]))

    def poll(self):
        try:
            command = [self.config['bin'], '-1']

            if str_to_bool(self.config['use_sudo']):
                command.insert(0, self.config['sudo_cmd'])

            output = subprocess.Popen(command,
                                      stdout=subprocess.PIPE).communicate()[0]
        except OSError:
            output = ""

        return output



# coding=utf-8

"""
Insert the collected values into a mysql table
"""

from handler import Handler
import MySQLdb


class MySQLHandler(Handler):
    """
    Implements the abstract Handler class, sending data to a mysql table
    """
    conn = None

    def __init__(self, config=None):
        """
        Create a new instance of the MySQLHandler class
        """
        # Initialize Handler
        Handler.__init__(self, config)

        # Initialize Options
        self.hostname = self.config['hostname']
        self.port = int(self.config['port'])
        self.username = self.config['username']
        self.password = self.config['password']
        self.database = self.config['database']
        self.table = self.config['table']
        self.col_time = self.config['col_time']
        self.col_metric = self.config['col_metric']
        self.col_value = self.config['col_value']

        # Connect
        self._connect()

    def get_default_config_help(self):
        """
        Returns the help text for the configuration options for this handler
        """
        config = super(MySQLHandler, self).get_default_config_help()

        config.update({
        })

        return config

    def get_default_config(self):
        """
        Return the default config for the handler
        """
        config = super(MySQLHandler, self).get_default_config()

        config.update({
        })

        return config

    def __del__(self):
        """
        Destroy instance of the MySQLHandler class
        """
        self._close()

    def process(self, metric):
        """
        Process a metric
        """
        # Just send the data
        self._send(str(metric))

    def _send(self, data):
        """
        Insert the data
        """
        data = data.strip().split(' ')
        try:
            cursor = self.conn.cursor()
            cursor.execute("INSERT INTO %s (%s, %s, %s) VALUES(%%s, %%s, %%s)"
                           % (self.table, self.col_metric,
                              self.col_time, self.col_value),
                           (data[0], data[2], data[1]))
            cursor.close()
            self.conn.commit()
            # Insert data
            pass
            #self.socket.sendall(data)
            # Done
        except BaseException, e:
            # Log Error
            self.log.error("MySQLHandler: Failed sending data. %s.", e)
            # Attempt to restablish connection
            self._connect()

    def _connect(self):
        """
        Connect to the MySQL server
        """
        self._close()
        self.conn = MySQLdb.Connect(host=self.hostname,
                                    port=self.port,
                                    user=self.username,
                                    passwd=self.password,
                                    db=self.database)

    def _close(self):
        """
        Close the connection
        """
        if self.conn:
            self.conn.commit()
            self.conn.close()



#!/usr/bin/python
# coding=utf-8
################################################################################

from test import CollectorTestCase
from test import get_collector_config
from test import unittest
from mock import Mock
from mock import patch

from diamond.collector import Collector
from ipvs import IPVSCollector

################################################################################


class TestIPVSCollector(CollectorTestCase):
    def setUp(self):
        config = get_collector_config('IPVSCollector', {
            'interval': 10,
            'bin': 'true',
            'use_sudo': False
        })

        self.collector = IPVSCollector(config, None)

    def test_import(self):
        self.assertTrue(IPVSCollector)

    @patch('os.access', Mock(return_value=True))
    @patch.object(Collector, 'publish')
    def test_should_work_with_real_data(self, publish_mock):
        patch_communicate = patch(
            'subprocess.Popen.communicate',
            Mock(return_value=(
                self.getFixture('ipvsadm').getvalue(),
                '')))

        patch_communicate.start()
        self.collector.collect()
        patch_communicate.stop()

        metrics = {
            "TCP_172_16_1_56:80.total.conns": 116,
            "TCP_172_16_1_56:443.total.conns": 59,
            "TCP_172_16_1_56:443.10_68_15_66:443.conns": 59,
            "TCP_172_16_1_56:443.10_68_15_66:443.outbytes": 216873,
        }

        self.setDocExample(collector=self.collector.__class__.__name__,
                           metrics=metrics,
                           defaultpath=self.collector.config['path'])
        self.assertPublishedMany(publish_mock, metrics)

################################################################################
if __name__ == "__main__":
    unittest.main()



# coding=utf-8

"""
This class collects data on memory utilization

Note that MemFree may report no memory free. This may not actually be the case,
as memory is allocated to Buffers and Cache as well. See
[this link](http://www.linuxatemyram.com/) for more details.

#### Dependencies

* /proc/meminfo or psutil

"""

import diamond.collector
import diamond.convertor
import os

try:
    import psutil
    psutil  # workaround for pyflakes issue #13
except ImportError:
    psutil = None

_KEY_MAPPING = [
    'MemTotal',
    'MemFree',
    'Buffers',
    'Cached',
    'Active',
    'Dirty',
    'Inactive',
    'Shmem',
    'SwapTotal',
    'SwapFree',
    'SwapCached',
    'VmallocTotal',
    'VmallocUsed',
    'VmallocChunk'
]


class MemoryCollector(diamond.collector.Collector):

    PROC = '/proc/meminfo'

    def get_default_config_help(self):
        config_help = super(MemoryCollector, self).get_default_config_help()
        config_help.update({
            'detailed': 'Set to True to Collect all the nodes',
        })
        return config_help

    def get_default_config(self):
        """
        Returns the default collector settings
        """
        config = super(MemoryCollector, self).get_default_config()
        config.update({
            'enabled':  'True',
            'path':     'memory',
            'method':   'Threaded',
            # Collect all the nodes or just a few standard ones?
            # Uncomment to enable
            #'detailed': 'True'
        })
        return config

    def collect(self):
        """
        Collect memory stats
        """
        if os.access(self.PROC, os.R_OK):
            file = open(self.PROC)
            data = file.read()
            file.close()

            for line in data.splitlines():
                try:
                    name, value, units = line.split()
                    name = name.rstrip(':')
                    value = int(value)

                    if (name not in _KEY_MAPPING
                            and 'detailed' not in self.config):
                        continue

                    for unit in self.config['byte_unit']:
                        value = diamond.convertor.binary.convert(value=value,
                                                                 oldUnit=units,
                                                                 newUnit=unit)
                        self.publish(name, value, metric_type='GAUGE')

                        # TODO: We only support one unit node here. Fix it!
                        break

                except ValueError:
                    continue
            return True
        else:
            if not psutil:
                self.log.error('Unable to import psutil')
                self.log.error('No memory metrics retrieved')
                return None

            phymem_usage = psutil.phymem_usage()
            virtmem_usage = psutil.virtmem_usage()
            units = 'B'

            for unit in self.config['byte_unit']:
                value = diamond.convertor.binary.convert(
                    value=phymem_usage.total, oldUnit=units, newUnit=unit)
                self.publish('MemTotal', value, metric_type='GAUGE')

                value = diamond.convertor.binary.convert(
                    value=phymem_usage.free, oldUnit=units, newUnit=unit)
                self.publish('MemFree', value, metric_type='GAUGE')

                value = diamond.convertor.binary.convert(
                    value=virtmem_usage.total, oldUnit=units, newUnit=unit)
                self.publish('SwapTotal', value, metric_type='GAUGE')

                value = diamond.convertor.binary.convert(
                    value=virtmem_usage.free, oldUnit=units, newUnit=unit)
                self.publish('SwapFree', value, metric_type='GAUGE')

                # TODO: We only support one unit node here. Fix it!
                break

            return True

        return None



# coding=utf-8

import logging
import threading
import traceback
from configobj import ConfigObj


class Handler(object):
    """
    Handlers process metrics that are collected by Collectors.
    """
    def __init__(self, config=None):
        """
        Create a new instance of the Handler class
        """

        # Initialize Log
        self.log = logging.getLogger('diamond')

        # Initialize Blank Configs
        self.config = ConfigObj()

        # Load default
        self.config.merge(self.get_default_config())

        # Load in user
        self.config.merge(config)

        # Initialize Lock
        self.lock = threading.Lock()

    def get_default_config_help(self):
        """
        Returns the help text for the configuration options for this handler
        """
        return {
            'get_default_config_help': 'get_default_config_help',
        }

    def get_default_config(self):
        """
        Return the default config for the handler
        """
        return {
            'get_default_config': 'get_default_config',
        }

    def _process(self, metric):
        """
        Decorator for processing handlers with a lock, catching exceptions
        """
        try:
            try:
                self.lock.acquire()
                self.process(metric)
            except Exception:
                self.log.error(traceback.format_exc())
        finally:
            if self.lock.locked():
                self.lock.release()

    def process(self, metric):
        """
        Process a metric

        Should be overridden in subclasses
        """
        raise NotImplementedError

    def _flush(self):
        """
        Decorator for flushing handlers with an lock, catching exceptions
        """
        try:
            try:
                self.lock.acquire()
                self.flush()
            except Exception:
                self.log.error(traceback.format_exc())
        finally:
            if self.lock.locked():
                self.lock.release()

    def flush(self):
        """
        Flush metrics

        Optional: Should be overridden in subclasses
        """
        pass



# coding=utf-8

"""
The MemoryCgroupCollector collects memory metric for cgroups

Stats that we are interested in tracking
cache - # of bytes of page cache memory.
rss   - # of bytes of anonymous and swap cache memory.
swap  - # of bytes of swap usage

#### Dependencies

/sys/fs/cgroup/memory/memory.stat
"""

import diamond.collector
import os

_KEY_MAPPING = [
    'cache',
    'rss',
    'swap'
]


class MemoryCgroupCollector(diamond.collector.Collector):
    MEMORY_PATH = '/sys/fs/cgroup/memory/'

    def get_default_config_help(self):
        config_help = super(
            MemoryCgroupCollector, self).get_default_config_help()
        config_help.update({
        })
        return config_help

    def get_default_config(self):
        """
        Returns the default collector settings
        """
        config = super(MemoryCgroupCollector, self).get_default_config()
        config.update({
            'path':     'memory_cgroup',
            'method':   'Threaded',
        })
        return config

    def collect(self):
        # find all memory.stat files
        matches = []
        for root, dirnames, filenames in os.walk(self.MEMORY_PATH):
            for filename in filenames:
                if filename == 'memory.stat':
                    # matches will contain a tuple contain path to cpuacct.stat
                    # and the parent of the stat
                    parent = root.replace(self.MEMORY_PATH,
                                          "").replace("/", ".")
                    if parent == '':
                        parent = 'system'
                    matches.append((parent, os.path.join(root, filename)))

        # Read metrics from cpuacct files
        results = {}
        for match in matches:
            results[match[0]] = {}
            stat_file = open(match[1])
            elements = [line.split() for line in stat_file]
            stat_file.close()

            for el in elements:
                name, value = el
                if name not in _KEY_MAPPING:
                    continue
                for unit in self.config['byte_unit']:
                    value = diamond.convertor.binary.convert(
                        value=value, oldUnit='B', newUnit=unit)
                    results[match[0]][name] = value
                    # TODO: We only support one unit node here. Fix it!
                    break

        # create metrics from collected utimes and stimes for cgroups
        for parent, cpuacct in results.iteritems():
            for key, value in cpuacct.iteritems():
                metric_name = '.'.join([parent, key])
                self.publish(metric_name, value, metric_type='GAUGE')
        return True



#!/usr/bin/python
# coding=utf-8
################################################################################
import os
from test import CollectorTestCase
from test import get_collector_config
from test import unittest
from mock import Mock
from mock import patch

try:
    from cStringIO import StringIO
    StringIO  # workaround for pyflakes issue #13
except ImportError:
    from StringIO import StringIO

from diamond.collector import Collector
from cpuacct_cgroup import CpuAcctCgroupCollector

dirname = os.path.dirname(__file__)
fixtures_path = os.path.join(dirname, 'fixtures/')
fixtures = []
for root, dirnames, filenames in os.walk(fixtures_path):
    fixtures.append([root, dirnames, filenames])


class TestCpuAcctCgroupCollector(CollectorTestCase):
    def setUp(self):
        config = get_collector_config('CpuAcctCgroupCollector', {
            'interval': 10
        })

        self.collector = CpuAcctCgroupCollector(config, None)

    def test_import(self):
        self.assertTrue(CpuAcctCgroupCollector)

    @patch('__builtin__.open')
    @patch('os.walk', Mock(return_value=iter(fixtures)))
    @patch.object(Collector, 'publish')
    def test_should_open_all_cpuacct_stat(self, publish_mock, open_mock):
        open_mock.side_effect = lambda x: StringIO('')
        self.collector.collect()
        open_mock.assert_any_call(
            fixtures_path + 'lxc/testcontainer/cpuacct.stat')
        open_mock.assert_any_call(fixtures_path + 'lxc/cpuacct.stat')
        open_mock.assert_any_call(fixtures_path + 'cpuacct.stat')

    @patch.object(Collector, 'publish')
    def test_should_work_with_real_data(self, publish_mock):
        CpuAcctCgroupCollector.CPUACCT_PATH = fixtures_path
        self.collector.collect()

        self.assertPublishedMany(publish_mock, {
            'lxc.testcontainer.user': 1318,
            'lxc.testcontainer.system': 332,
            'lxc.user': 36891,
            'lxc.system': 88927,
            'system.user': 3781253,
            'system.system': 4784004,
        })

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python
# coding=utf-8
################################################################################

from test import CollectorTestCase
from test import get_collector_config
from test import unittest
from mock import Mock
from mock import patch

from diamond.collector import Collector
from squid import SquidCollector

################################################################################


class TestSquidCollector(CollectorTestCase):
    def setUp(self):
        config = get_collector_config('SquidCollector', {
            'interval': 1,
        })

        self.collector = SquidCollector(config, None)

    def test_import(self):
        self.assertTrue(SquidCollector)

    @patch.object(Collector, 'publish')
    def test_should_work_with_fake_data(self, publish_mock):
        _getData_mock = patch.object(
            SquidCollector,
            '_getData',
            Mock(
                return_value=self.getFixture('fake_counters_1').getvalue()))
        _getData_mock.start()
        self.collector.collect()
        _getData_mock.stop()

        self.assertPublishedMany(publish_mock, {})

        _getData_mock = patch.object(
            SquidCollector,
            '_getData',
            Mock(
                return_value=self.getFixture('fake_counters_2').getvalue()))
        _getData_mock.start()
        self.collector.collect()
        _getData_mock.stop()

        metrics = {
            '3128.client_http.requests': 1,
            '3128.client_http.hits': 2,
            '3128.client_http.errors': 3,
            '3128.client_http.kbytes_in': 4,
            '3128.client_http.kbytes_out': 5,
            '3128.client_http.hit_kbytes_out': 6,
            '3128.server.all.requests': 7,
            '3128.server.all.errors': 8,
            '3128.server.all.kbytes_in': 9,
            '3128.server.all.kbytes_out': 10,
            '3128.server.http.requests': 1,
            '3128.server.http.errors': 12,
            '3128.server.http.kbytes_in': 13,
            '3128.server.http.kbytes_out': 14,
            '3128.server.ftp.requests': 15,
            '3128.server.ftp.errors': 16,
            '3128.server.ftp.kbytes_in': 17,
            '3128.server.ftp.kbytes_out': 18,
            '3128.server.other.requests': 19,
            '3128.server.other.errors': 20,
            '3128.server.other.kbytes_in': 21,
            '3128.server.other.kbytes_out': 22,
            '3128.icp.pkts_sent': 23,
            '3128.icp.pkts_recv': 24,
            '3128.icp.queries_sent': 25,
            '3128.icp.replies_sent': 26,
            '3128.icp.queries_recv': 27,
            '3128.icp.replies_recv': 28,
            '3128.icp.query_timeouts': 29,
            '3128.icp.replies_queued': 30,
            '3128.icp.kbytes_sent': 31,
            '3128.icp.kbytes_recv': 32,
            '3128.icp.q_kbytes_sent': 33,
            '3128.icp.r_kbytes_sent': 34,
            '3128.icp.q_kbytes_recv': 35,
            '3128.icp.r_kbytes_recv': 36,
            '3128.icp.times_used': 37,
            '3128.cd.times_used': 38,
            '3128.cd.msgs_sent': 39,
            '3128.cd.msgs_recv': 40,
            '3128.cd.memory': 41,
            '3128.cd.local_memory': 42,
            '3128.cd.kbytes_sent': 43,
            '3128.cd.kbytes_recv': 44,
            '3128.unlink.requests': 45,
            '3128.page_faults': 46,
            '3128.select_loops': 47,
            '3128.cpu_time': 48.1234567890,
            '3128.wall_time': 49.1234567890,
            '3128.swap.outs': 50,
            '3128.swap.ins': 51,
            '3128.swap.files_cleaned': 52,
            '3128.aborted_requests': 53
        }

        self.assertPublishedMany(publish_mock, metrics)

    @patch.object(Collector, 'publish')
    def test_should_work_with_real_data(self, publish_mock):
        _getData_mock = patch.object(
            SquidCollector,
            '_getData',
            Mock(
                return_value=self.getFixture('counters_1').getvalue()))
        _getData_mock.start()
        self.collector.collect()
        _getData_mock.stop()

        self.assertPublishedMany(publish_mock, {})

        _getData_mock = patch.object(
            SquidCollector,
            '_getData',
            Mock(
                return_value=self.getFixture('counters_2').getvalue()))
        _getData_mock.start()
        self.collector.collect()
        _getData_mock.stop()

        metrics = {
            '3128.client_http.requests': 2,
            '3128.client_http.hits': 1,
            '3128.client_http.errors': 0,
            '3128.client_http.kbytes_in': 1,
            '3128.client_http.kbytes_out': 12.0,
            '3128.client_http.hit_kbytes_out': 10,
            '3128.server.all.requests': 0,
            '3128.server.all.errors': 0,
            '3128.server.all.kbytes_in': 0,
            '3128.server.all.kbytes_out': 0,
            '3128.server.http.requests': 0,
            '3128.server.http.errors': 0,
            '3128.server.http.kbytes_in': 0,
            '3128.server.http.kbytes_out': 0,
            '3128.server.ftp.requests': 0,
            '3128.server.ftp.errors': 0,
            '3128.server.ftp.kbytes_in': 0,
            '3128.server.ftp.kbytes_out': 0,
            '3128.server.other.requests': 0,
            '3128.server.other.errors': 0,
            '3128.server.other.kbytes_in': 0,
            '3128.server.other.kbytes_out': 0,
            '3128.icp.pkts_sent': 0,
            '3128.icp.pkts_recv': 0,
            '3128.icp.queries_sent': 0,
            '3128.icp.replies_sent': 0,
            '3128.icp.queries_recv': 0,
            '3128.icp.replies_recv': 0,
            '3128.icp.query_timeouts': 0,
            '3128.icp.replies_queued': 0,
            '3128.icp.kbytes_sent': 0,
            '3128.icp.kbytes_recv': 0,
            '3128.icp.q_kbytes_sent': 0,
            '3128.icp.r_kbytes_sent': 0,
            '3128.icp.q_kbytes_recv': 0,
            '3128.icp.r_kbytes_recv': 0,
            '3128.icp.times_used': 0,
            '3128.cd.times_used': 0,
            '3128.cd.msgs_sent': 0,
            '3128.cd.msgs_recv': 0,
            '3128.cd.memory': 0,
            '3128.cd.local_memory': 0,
            '3128.cd.kbytes_sent': 0,
            '3128.cd.kbytes_recv': 0,
            '3128.unlink.requests': 0,
            '3128.page_faults': 0,
            '3128.select_loops': 10827.0,
            '3128.cpu_time': 0,
            '3128.wall_time': 10,
            '3128.swap.outs': 0,
            '3128.swap.ins': 2,
            '3128.swap.files_cleaned': 0,
            '3128.aborted_requests': 0
        }

        self.setDocExample(collector=self.collector.__class__.__name__,
                           metrics=metrics,
                           defaultpath=self.collector.config['path'])
        self.assertPublishedMany(publish_mock, metrics)

################################################################################
if __name__ == "__main__":
    unittest.main()



# coding=utf-8

"""
The InterruptCollector class collects metrics on interrupts from
/proc/interrupts

#### Dependencies

 * /proc/interrupts

"""

import platform
import os
import diamond.collector

# Detect the architecture of the system
# and set the counters for MAX_VALUES
# appropriately. Otherwise, rolling over
# counters will cause incorrect or
# negative values.
if platform.architecture()[0] == '64bit':
    counter = (2 ** 64) - 1
else:
    counter = (2 ** 32) - 1


class InterruptCollector(diamond.collector.Collector):

    PROC = '/proc/interrupts'

    def get_default_config_help(self):
        config_help = super(InterruptCollector, self).get_default_config_help()
        config_help.update({
        })
        return config_help

    def get_default_config(self):
        """
        Returns the default collector settings
        """
        config = super(InterruptCollector, self).get_default_config()
        config.update({
            'path':     'interrupts'
        })
        return config

    def collect(self):
        """
        Collect interrupt data
        """
        if not os.access(self.PROC, os.R_OK):
            return False

        #Open PROC file
        file = open(self.PROC, 'r')
        #Get data
        cpuCount = None
        for line in file:
            if not cpuCount:
                cpuCount = len(line.split())
            else:
                data = line.strip().split(None, cpuCount + 2)
                data[0] = data[0].replace(':', '')

                if len(data) == 2:
                    metric_name = data[0]
                    metric_value = data[1]
                    self.publish(metric_name,
                                 self.derivative(metric_name,
                                                 long(metric_value),
                                                 counter))
                else:
                    if len(data[0]) == cpuCount + 1:
                        metric_name = data[0] + '.'
                    elif len(data[0]) == 3:
                        metric_name = (((data[-2] + ' '
                                         + data[-1]).replace(' ', '_')) + '.')
                    else:
                        metric_name = (((data[-2]).replace(' ', '_'))
                                       + '.'
                                       + ((data[-1]).replace(', ',
                                                             '-'
                                                             ).replace(' ',
                                                                       '_'))
                                       + '.' + data[0] + '.')
                    total = 0
                    for index, value in enumerate(data):
                        if index == 0 or index >= cpuCount + 1:
                            continue

                        metric_name_node = metric_name + 'CPU' + str(index - 1)
                        value = int(self.derivative(metric_name_node,
                                                    long(value), counter))
                        total += value
                        self.publish(metric_name_node, value)

                    # Roll up value
                    metric_name_node = metric_name + 'total'
                    self.publish(metric_name_node, total)

        #Close file
        file.close()



# coding=utf-8

import sys
import time
import re
import unicodedata

from diamond.metric import Metric
import diamond.convertor

"""
The NetAppCollector collects metric from a NetApp installation using the
NetApp Manageability SDK. This allows access to many metrics not available
via SNMP.

For this to work you'll the SDK available on the system.
This module has been developed using v5.0 of the SDK.
As of writing the SDK can be found at
https://communities.netapp.com/docs/DOC-1152

You'll also need to specify which NetApp instances the collecter should
get data from.

Example NetAppCollector.conf:
```
    enabled = True
    path_prefix = netapp

    [devices]

    [[na_filer]]
    ip = 123.123.123.123
    user = root
    password = strongpassword

````

The primary source for documentation about the API has been
"NetApp unified storage performance management using open interfaces"
https://communities.netapp.com/docs/DOC-1044

"""


class NetAppCollector(diamond.collector.Collector):

    # This is the list of metrics to collect.
    # This is a dict of lists with tuples, which is parsed as such:
    # The dict name is the object name in the NetApp API.
    # For each object we have a list of metrics to retrieve.
    # Each tuple is built like this;
    # ("metric name in netapp api", "output name of metric", multiplier)
    # The purpose of the output name is to enable replacement of reported
    # metric names, since some the names in the API can be confusing.
    # The purpose of the multiplier is to scale all metrics to a common
    # scale, which is latencies in milliseconds, and data in bytes per sec.
    # This is needed since the API will return a mixture of percentages,
    # nanoseconds, milliseconds, bytes and kilobytes.
    METRICS = {
        'aggregate': [
            ("user_reads", "user_read_iops", 1),
            ("user_writes", "user_write_iops", 1)
            ],
        'disk': [
            ("disk_busy", "disk_busy_pct", 100),
            ("base_for_disk_busy", "base_for_disk_busy", 1),
            ("user_read_blocks", "user_read_blocks_per_sec", 1),
            ("user_write_blocks", "user_write_blocks_per_sec", 1),
            ("user_read_latency", "user_read_latency", 0.001),
            ("user_write_latency", "user_write_latency", 0.001)
            ],
        'ifnet': [
            ("send_data", "tx_bytes_per_sec", 1),
            ("recv_data", "rx_bytes_per_sec", 1)
            ],
        'lun': [
            ("total_ops", "total_iops", 1),
            ("read_ops", "read_iops", 1),
            ("write_ops", "write_iops", 1),
            ("avg_latency", "avg_latency", 1)
            ],
        'processor': [
            ("processor_busy", "processor_busy_pct", 100),
            ("processor_elapsed_time", "processor_elapsed_time", 1)
            ],
        'system': [
            ("nfs_ops", "nfs_iops", 1),
            ("cifs_ops", "cifs_iops", 1),
            ("http_ops", "http_iops", 1),
            ("fcp_ops", "fcp_iops", 1),
            ("http_ops", "http_iops", 1),
            ("iscsi_ops", "iscsi_iops", 1),
            ("read_ops", "read_iops", 1),
            ("write_ops", "write_iops", 1),
            ("total_ops", "total_iops", 1),
            ("cpu_elapsed_time", "cpu_elapsed_time", 1),
            ("total_processor_busy", "total_processor_busy_pct", 100),
            ("avg_processor_busy", "avg_processor_busy_pct", 100),
            ("net_data_recv", "total_rx_bytes_per_sec", 1000),
            ("net_data_sent", "total_tx_bytes_per_sec", 1000),
            ("disk_data_read", "total_read_bytes_per_sec", 1000),
            ("disk_data_written", "total_write_bytes_per_sec", 1000),
            ("sys_read_latency", "sys_read_latency", 1),
            ("sys_write_latency", "sys_write_latency", 1),
            ("sys_avg_latency", "sys_avg_latency", 1)
            ],
        'vfiler': [
            ("vfiler_cpu_busy", "cpu_busy_pct", 100),
            ("vfiler_cpu_busy_base", "cpu_busy_base", 1),
            ("vfiler_net_data_recv", "rx_bytes_per_sec", 1000),
            ("vfiler_net_data_sent", "tx_bytes_per_sec", 1000),
            ("vfiler_read_ops", "read_iops", 1),
            ("vfiler_write_ops", "write_iops", 1),
            ("vfiler_read_bytes", "read_bytes_per_sec", 1000),
            ("vfiler_write_bytes", "write_bytes_per_sec", 1000),
            ],
        'volume': [
            ("total_ops", "total_iops", 1),
            ("avg_latency", "avg_latency", 0.001),
            ("read_ops", "read_iops", 1),
            ("write_ops", "write_iops", 1),
            ("read_latency", "read_latency", 0.001),
            ("write_latency", "write_latency", 0.001),
            ("read_data", "read_bytes_per_sec", 1),
            ("write_data", "write_bytes_per_sec", 1),
            ("cifs_read_data", "cifs_read_bytes_per_sec", 1),
            ("cifs_write_data", "cifs_write_bytes_per_sec", 1),
            ("cifs_read_latency", "cifs_read_latency", 0.001),
            ("cifs_write_latency", "cifs_write_latency", 0.001),
            ("cifs_read_ops", "cifs_read_iops", 1),
            ("cifs_write_ops", "cifs_write_iops", 1),
            ("fcp_read_data", "fcp_read_bytes_per_sec", 1),
            ("fcp_write_data", "fcp_write_bytes_per_sec", 1),
            ("fcp_read_latency", "fcp_read_latency", 0.001),
            ("fcp_write_latency", "fcp_write_latency", 0.001),
            ("fcp_read_ops", "fcp_read_iops", 1),
            ("fcp_write_ops", "fcp_write_iops", 1),
            ("iscsi_read_data", "iscsi_read_bytes_per_sec", 1),
            ("iscsi_write_data", "iscsi_write_bytes_per_sec", 1),
            ("iscsi_read_latency", "iscsi_read_latency", 0.001),
            ("iscsi_write_latency", "iscsi_write_latency", 0.001),
            ("iscsi_read_ops", "iscsi_read_iops", 1),
            ("iscsi_write_ops", "iscsi_write_iops", 1),
            ("nfs_read_data", "nfs_read_bytes_per_sec", 1),
            ("nfs_write_data", "nfs_write_bytes_per_sec", 1),
            ("nfs_read_latency", "nfs_read_latency", 0.001),
            ("nfs_write_latency", "nfs_write_latency", 0.001),
            ("nfs_read_ops", "nfs_read_iops", 1),
            ("nfs_write_ops", "nfs_write_iops", 1)
            ],
    }

    # For some metrics we need to divide one value from the API with another.
    # This is a key-value list of the connected values.
    DIVIDERS = {
        "avg_latency": "total_ops",
        "read_latency": "read_ops",
        "write_latency": "write_ops",
        "sys_avg_latency": "total_ops",
        "sys_read_latency": "read_ops",
        "sys_write_latency": "write_ops",
        "cifs_read_latency": "cifs_read_ops",
        "cifs_write_latency": "cifs_write_ops",
        "fcp_read_latency": "fcp_read_ops",
        "fcp_write_latency": "fcp_write_ops",
        "iscsi_read_latency": "iscsi_read_ops",
        "iscsi_write_latency": "iscsi_write_ops",
        "nfs_read_latency": "nfs_read_ops",
        "nfs_write_latency": "nfs_write_ops",
        "user_read_latency": "user_read_blocks",
        "user_write_latency": "user_write_blocks",
        "total_processor_busy": "cpu_elapsed_time",
        "avg_processor_busy": "cpu_elapsed_time",
        "processor_busy": "processor_elapsed_time",
        "disk_busy": "base_for_disk_busy",
        "vfiler_cpu_busy": "vfiler_cpu_busy_base",
    }

    # Some metrics are collected simply to calculate other metrics.
    # These should not be reported.
    DROPMETRICS = [
        "cpu_elapsed_time",
        "processor_elapsed_time",
        "base_for_disk_busy",
        "vfiler_cpu_busy_base",
    ]

    # Since we might have large collections collected often,
    # we need a pretty good time_delta.
    # We'll use a dict for this, keeping time_delta for each object.
    LastCollectTime = {}

    def get_default_config_help(self):
        config_help = super(NetAppCollector, self).get_default_config_help()
        return config_help

    def get_default_config(self):
        default_config = super(NetAppCollector, self).get_default_config()
        default_config['enabled'] = "false"
        default_config['path_prefix'] = "netapp"
        default_config['netappsdkpath'] = "/opt/netapp/lib/python/NetApp"
        return default_config

    def get_schedule(self):
        """
        Override Collector.get_schedule, and create one per filer.
        """
        schedule = {}
        if 'devices' in self.config:
            for device in self.config['devices']:
                # Get Device Config
                c = self.config['devices'][device]
                # Get Task Name
                task = "_".join([self.__class__.__name__, device])
                # Check if task is already in schedule
                if task in schedule:
                    raise KeyError("Duplicate device scheduled")
                schedule[task] = (self.collect,
                                  (device, c['ip'], c['user'], c['password']),
                                  int(self.config['splay']),
                                  int(self.config['interval']))
        self.log.info("Set up scheduler for %s" % device)
        return schedule

    def _replace_and_publish(self, path, prettyname, value, device):
        """
        Inputs a complete path for a metric and a value.
        Replace the metric name and publish.
        """
        if value is None:
            return
        newpath = path
        # Change metric name before publish if needed.
        newpath = ".".join([".".join(path.split(".")[:-1]), prettyname])
        metric = Metric(newpath, value, precision=4, host=device)
        self.publish_metric(metric)

    def _gen_delta_depend(self, path, derivative, multiplier, prettyname,
                          device):
        """
        For some metrics we need to divide the delta for one metric
        with the delta of another.
        Publishes a metric if the convertion goes well.
        """
        primary_delta = derivative[path]
        shortpath = ".".join(path.split(".")[:-1])
        basename = path.split(".")[-1]
        secondary_delta = None
        if basename in self.DIVIDERS.keys():
            mateKey = ".".join([shortpath, self.DIVIDERS[basename]])
        else:
            return
        if mateKey in derivative.keys():
            secondary_delta = derivative[mateKey]
        else:
            return

        # If we find a corresponding secondary_delta, publish a metric
        if primary_delta > 0 and secondary_delta > 0:
            value = (float(primary_delta) / secondary_delta) * multiplier
            self._replace_and_publish(path, prettyname, value, device)

    def _gen_delta_per_sec(self, path, value_delta, time_delta, multiplier,
                           prettyname, device):
        """
        Calulates the difference between to point, and scales is to per second.
        """
        if time_delta < 0:
            return
        value = (value_delta / time_delta) * multiplier
        # Only publish if there is any data.
        # This helps keep unused metrics out of Graphite
        if value > 0.0:
            self._replace_and_publish(path, prettyname, value, device)

    def collect(self, device, ip, user, password):
        """
        This function collects the metrics for one filer.
        """
        sys.path.append(self.config['netappsdkpath'])
        try:
            import NaServer
        except ImportError:
            self.log.error("Unable to load NetApp SDK from %s" % (
                self.config['netappsdkpath']))
            return

        # Set up the parameters
        server = NaServer.NaServer(ip, 1, 3)
        server.set_transport_type('HTTPS')
        server.set_style('LOGIN')
        server.set_admin_user(user, password)

        # We're only able to query a single object at a time,
        # so we'll loop over the objects.
        for na_object in self.METRICS.keys():

            # For easy reference later, generate a new dict for this object
            LOCALMETRICS = {}
            for metric in self.METRICS[na_object]:
                metricname, prettyname, multiplier = metric
                LOCALMETRICS[metricname] = {}
                LOCALMETRICS[metricname]["prettyname"] = prettyname
                LOCALMETRICS[metricname]["multiplier"] = multiplier

            # Keep track of how long has passed since we checked last
            CollectTime = time.time()
            time_delta = None
            if na_object in self.LastCollectTime.keys():
                time_delta = CollectTime - self.LastCollectTime[na_object]
            self.LastCollectTime[na_object] = CollectTime

            self.log.debug("Collecting metric of object %s" % na_object)
            query = NaServer.NaElement("perf-object-get-instances-iter-start")
            query.child_add_string("objectname", na_object)
            counters = NaServer.NaElement("counters")
            for metric in LOCALMETRICS.keys():
                counters.child_add_string("counter", metric)
            query.child_add(counters)

            res = server.invoke_elem(query)
            if(res.results_status() == "failed"):
                self.log.error("Connection to filer %s failed; %s" % (
                    device, res.results_reason()))
                return

            iter_tag = res.child_get_string("tag")
            num_records = 1
            max_records = 100

            # For some metrics there are dependencies between metrics for
            # a single object, so we'll need to collect all, so we can do
            # calculations later.
            raw = {}

            while(num_records != 0):
                query = NaServer.NaElement(
                    "perf-object-get-instances-iter-next")
                query.child_add_string("tag", iter_tag)
                query.child_add_string("maximum", max_records)
                res = server.invoke_elem(query)

                if(res.results_status() == "failed"):
                    print "Connection to filer %s failed; %s" % (
                        device, res.results_reason())
                    return

                num_records = res.child_get_int("records")

                if(num_records > 0):
                    instances_list = res.child_get("instances")
                    instances = instances_list.children_get()

                    for instance in instances:
                        raw_name = unicodedata.normalize(
                            'NFKD',
                            instance.child_get_string("name")).encode(
                            'ascii', 'ignore')
                        # Shorten the name for disks as they are very long and
                        # padded with zeroes, eg:
                        # 5000C500:3A236B0B:00000000:00000000:00000000:...
                        if na_object is "disk":
                            non_zero_blocks = [
                                block for block in raw_name.split(":")
                                if block != "00000000"
                                ]
                            raw_name = "".join(non_zero_blocks)
                        instance_name = re.sub(r'\W', '_', raw_name)
                        counters_list = instance.child_get("counters")
                        counters = counters_list.children_get()

                        for counter in counters:
                            metricname = unicodedata.normalize(
                                'NFKD',
                                counter.child_get_string("name")).encode(
                                'ascii', 'ignore')
                            metricvalue = counter.child_get_string("value")
                            # We'll need a long complete pathname to not
                            # confuse self.derivative
                            pathname = ".".join([self.config["path_prefix"],
                                                 device, na_object,
                                                 instance_name, metricname])
                            raw[pathname] = int(metricvalue)

            # Do the math
            self.log.debug("Processing %i metrics for object %s" % (len(raw),
                                                                    na_object))

            # Since the derivative function both returns the derivative
            # and saves a new point, we'll need to store all derivatives
            # for local reference.
            derivative = {}
            for key in raw.keys():
                derivative[key] = self.derivative(key, raw[key])

            for key in raw.keys():
                metricname = key.split(".")[-1]
                prettyname = LOCALMETRICS[metricname]["prettyname"]
                multiplier = LOCALMETRICS[metricname]["multiplier"]

                if metricname in self.DROPMETRICS:
                    continue
                elif metricname in self.DIVIDERS.keys():
                    self._gen_delta_depend(key, derivative, multiplier,
                                           prettyname, device)
                else:
                    self._gen_delta_per_sec(key, derivative[key], time_delta,
                                            multiplier, prettyname, device)



# coding=utf-8

"""
The PasengerCollector collects CPU and memory utilization of apache, nginx
and passenger processes.

Four key attributes to be published:
phusion_passenger_cpu, total_apache_memory, total_passenger_mem, total_nginx_mem

#### Dependencies

 * passenger-memory-stats

"""
import diamond.collector
import os
import re
import subprocess
from diamond.collector import str_to_bool
from cStringIO import StringIO


class PassengerCollector(diamond.collector.Collector):

    def whereis(self, program):
        for path in os.environ.get('PATH', '').split(':'):
            if os.path.exists(os.path.join(path, program)) and \
               not os.path.isdir(os.path.join(path, program)):
                return os.path.join(path, program)
        return None

    def get_default_config_help(self):
        config_help = super(PassengerCollector, self).get_default_config_help()
        config_help.update({
            'bin':         'The path to the binary',
            'use_sudo':    'Use sudo?',
            'sudo_cmd':    'Path to sudo',
        })
        return config_help

    def get_default_config(self):
        """
        Returns the default collector settings
        """
        config = super(PassengerCollector, self).get_default_config()
        config.update({
            'path':         'passenger_stats',
            'bin':          '/usr/lib/ruby-flo/bin/passenger-memory-stats',
            'use_sudo':     False,
            'sudo_cmd':     '/usr/bin/sudo',
            })
        return config

    def collect(self):
        """
        Collector Passenger stats
        """

        # If the config path doesn't exist, scan for one
        if not os.access(self.config['bin'], os.X_OK):
            self.config['bin'] = self.whereis('passenger-memory-stats')

        if not os.access(self.config['bin'], os.X_OK):
            self.log.error("Path %s does not exist or is not executable"
                           % self.config['bin'])
            return

        command = [self.config['bin']]

        if str_to_bool(self.config['use_sudo']):
            command.insert(0, self.config['sudo_cmd'])

        def calc_cpu(processes):
            pipe1 = "top -b -n 2"
            pipe2 = "egrep " + processes
            pipe3 = "awk {print $9}"
            p1 = subprocess.Popen(pipe1.split(), stdout=subprocess.PIPE)
            p2 = subprocess.Popen(pipe2.split(), stdin=p1.stdout,
                                  stdout=subprocess.PIPE)
            p3 = subprocess.Popen(pipe3.split(" ", 1), stdin=p2.stdout,
                                  stdout=subprocess.PIPE)
            result = p3.communicate()
            cpu_usage = result[0].split('\n')[:-1]
            total_cpu = 0.0
            for index in range(len(cpu_usage) / 2, len(cpu_usage)):
                cpu = float(cpu_usage[index])
                total_cpu += cpu

            return str(total_cpu)

        k2 = 'sed -r s/\x1B\[([0-9]{1,3}((;[0-9]{1,3})*)?)?[m|K]//g'
        q1 = subprocess.Popen(command, stdout=subprocess.PIPE)
        q2 = subprocess.Popen(k2.split(), stdin=q1.stdout,
                              stdout=subprocess.PIPE)
        (res, err) = q2.communicate()
        f = StringIO(res)
        passenger_flag = 0
        apache_flag = 0
        nginx_flag = 0

        apache_processes = ""
        nginx_processes = ""
        passenger_processes = ""

        total_passenger_mem = 0.0
        total_apache_mem = 0.0
        total_nginx_mem = 0.0
        for line in f:
            if('Passenger processes' in line):
                passenger_flag = 1
                continue
            if('Apache processes' in line):
                apache_flag = 1
                continue
            if('Nginx processes' in line):
                nginx_flag = 1
                continue

            #If line starts with digit, then store process ids and memory
            matchObj = re.match(r"^\d", line)
            if matchObj:
                processList = line.split()
                if(apache_flag == 1):
                    apache_processes += (processList[0] + "|")
                    total_apache_mem += float(processList[4])
                if(passenger_flag == 1):
                    passenger_processes += (processList[0] + "|")
                    total_passenger_mem += float(processList[3])
                if(nginx_flag == 1):
                    nginx_processes += (processList[0] + "|")
                    total_nginx_mem += float(processList[4])

            elif('Processes:' in line):
                passenger_flag = 0
                apache_flag = 0
                nginx_flag = 0
        apache_processes = apache_processes[:-1]
        nginx_processes = nginx_processes[:-1]
        passenger_processes = passenger_processes[:-1]

        #calculate cpu of apache, nginx and passenger processes
        overall_cpu = 0
        if apache_processes:
            apache_cpu = calc_cpu(apache_processes)
            overall_cpu += float(apache_cpu)
        if nginx_processes:
            nginx_cpu = calc_cpu(nginx_processes)
            overall_cpu += float(nginx_cpu)
        if passenger_processes:
            passenger_cpu = calc_cpu(passenger_processes)
            overall_cpu += float(passenger_cpu)

        self.publish('phusion_passenger_cpu', overall_cpu)
        self.publish('total_apache_memory', total_apache_mem)
        self.publish('total_passenger_mem', total_passenger_mem)
        self.publish('total_nginx_mem', total_nginx_mem)



#!/usr/bin/python
# coding=utf-8
################################################################################

from test import CollectorTestCase
from test import get_collector_config
from test import unittest
from mock import Mock
from mock import patch

from diamond.collector import Collector
from powerdns import PowerDNSCollector

################################################################################


class TestPowerDNSCollector(CollectorTestCase):
    def setUp(self):
        config = get_collector_config('PowerDNSCollector', {
            'interval': 1,
            'bin': 'true',
            'use_sudo': False,
        })

        self.collector = PowerDNSCollector(config, None)

    def test_import(self):
        self.assertTrue(PowerDNSCollector)

    @patch('os.access', Mock(return_value=True))
    @patch.object(Collector, 'publish')
    def test_should_work_with_fake_data(self, publish_mock):
        patch_communicate = patch(
            'subprocess.Popen.communicate',
            Mock(return_value=(
                self.getFixture(
                    'pdns_control-2.9.22.6-1.el6-A'
                    ).getvalue(),
                '')))

        patch_communicate.start()
        self.collector.collect()
        patch_communicate.stop()

        self.assertPublishedMany(publish_mock, {})

        patch_communicate = patch(
            'subprocess.Popen.communicate',
            Mock(return_value=(
                self.getFixture('pdns_control-2.9.22.6-1.el6-B').getvalue(),
                '')))

        patch_communicate.start()
        self.collector.collect()
        patch_communicate.stop()

        metrics = {
            'corrupt-packets': 1.0,
            'deferred-cache-inserts': 2.0,
            'deferred-cache-lookup': 3.0,
            'latency': 4.0,
            'packetcache-hit': 5.0,
            'packetcache-miss': 6.0,
            'packetcache-size': 7.0,
            'qsize-q': 8.0,
            'query-cache-hit': 9.0,
            'query-cache-miss': 10.0,
            'recursing-answers': 11.0,
            'recursing-questions': 12.0,
            'servfail-packets': 13.0,
            'tcp-answers': 14.0,
            'tcp-queries': 15.0,
            'timedout-packets': 16.0,
            'udp-answers': 17.0,
            'udp-queries': 18.0,
            'udp4-answers': 19.0,
            'udp4-queries': 20.0,
            'udp6-answers': 21.0,
            'udp6-queries': 22.0,
        }

        self.setDocExample(collector=self.collector.__class__.__name__,
                           metrics=metrics,
                           defaultpath=self.collector.config['path'])
        self.assertPublishedMany(publish_mock, metrics)

################################################################################
if __name__ == "__main__":
    unittest.main()



# coding=utf-8

"""
The CPUCollector collects CPU utilization metric using /proc/stat.

#### Dependencies

 * /proc/stat

"""

import diamond.collector
import os
import time
from diamond.collector import str_to_bool

try:
    import psutil
    psutil  # workaround for pyflakes issue #13
except ImportError:
    psutil = None


class CPUCollector(diamond.collector.Collector):

    PROC = '/proc/stat'
    INTERVAL = 1

    MAX_VALUES = {
        'user': diamond.collector.MAX_COUNTER,
        'nice': diamond.collector.MAX_COUNTER,
        'system': diamond.collector.MAX_COUNTER,
        'idle': diamond.collector.MAX_COUNTER,
        'iowait': diamond.collector.MAX_COUNTER,
        'irq': diamond.collector.MAX_COUNTER,
        'softirq': diamond.collector.MAX_COUNTER,
        'steal': diamond.collector.MAX_COUNTER,
        'guest': diamond.collector.MAX_COUNTER,
        'guest_nice': diamond.collector.MAX_COUNTER,
    }

    def get_default_config_help(self):
        config_help = super(CPUCollector, self).get_default_config_help()
        config_help.update({
            'percore':  'Collect metrics per cpu core or just total',
            'simple':   'only return aggregate CPU% metric',
            'normalize': 'for cpu totals, divide by the number of CPUs',
        })
        return config_help

    def get_default_config(self):
        """
        Returns the default collector settings
        """
        config = super(CPUCollector, self).get_default_config()
        config.update({
            'enabled':  'True',
            'path':     'cpu',
            'percore':  'True',
            'xenfix':   None,
            'simple':   'False',
            'normalize': 'False',
        })
        return config

    def collect(self):
        """
        Collector cpu stats
        """

        def cpu_time_list():
            """
            get cpu time list
            """

            statFile = open(self.PROC, "r")
            timeList = statFile.readline().split(" ")[2:6]
            for i in range(len(timeList)):
                timeList[i] = int(timeList[i])
            statFile.close()
            return timeList

        def cpu_delta_time(interval):
            """
            Get before and after cpu times for usage calc
            """
            pre_check = cpu_time_list()
            time.sleep(interval)
            post_check = cpu_time_list()
            for i in range(len(pre_check)):
                post_check[i] -= pre_check[i]
            return post_check

        if os.access(self.PROC, os.R_OK):

            #If simple only return aggregate CPU% metric
            if str_to_bool(self.config['simple']):
                dt = cpu_delta_time(self.INTERVAL)
                cpuPct = 100 - (dt[len(dt) - 1] * 100.00 / sum(dt))
                self.publish('percent', str('%.4f' % cpuPct))
                return True

            results = {}
            # Open file
            file = open(self.PROC)

            ncpus = -1  # dont want to count the 'cpu'(total) cpu.
            for line in file:
                if not line.startswith('cpu'):
                    continue

                ncpus += 1
                elements = line.split()

                cpu = elements[0]

                if cpu == 'cpu':
                    cpu = 'total'
                elif not str_to_bool(self.config['percore']):
                    continue

                results[cpu] = {}

                if len(elements) >= 2:
                    results[cpu]['user'] = elements[1]
                if len(elements) >= 3:
                    results[cpu]['nice'] = elements[2]
                if len(elements) >= 4:
                    results[cpu]['system'] = elements[3]
                if len(elements) >= 5:
                    results[cpu]['idle'] = elements[4]
                if len(elements) >= 6:
                    results[cpu]['iowait'] = elements[5]
                if len(elements) >= 7:
                    results[cpu]['irq'] = elements[6]
                if len(elements) >= 8:
                    results[cpu]['softirq'] = elements[7]
                if len(elements) >= 9:
                    results[cpu]['steal'] = elements[8]
                if len(elements) >= 10:
                    results[cpu]['guest'] = elements[9]
                if len(elements) >= 11:
                    results[cpu]['guest_nice'] = elements[10]

            # Close File
            file.close()

            metrics = {}

            for cpu in results.keys():
                stats = results[cpu]
                for s in stats.keys():
                    # Get Metric Name
                    metric_name = '.'.join([cpu, s])
                    # Get actual data
                    if (str_to_bool(self.config['normalize'])
                            and cpu == 'total' and ncpus > 0):
                        metrics[metric_name] = self.derivative(
                            metric_name,
                            long(stats[s]),
                            self.MAX_VALUES[s]) / ncpus
                    else:
                        metrics[metric_name] = self.derivative(
                            metric_name,
                            long(stats[s]),
                            self.MAX_VALUES[s])

            # Check for a bug in xen where the idle time is doubled for guest
            # See https://bugzilla.redhat.com/show_bug.cgi?id=624756
            if self.config['xenfix'] is None or self.config['xenfix'] is True:
                if os.path.isdir('/proc/xen'):
                    total = 0
                    for metric_name in metrics.keys():
                        if 'cpu0.' in metric_name:
                            total += int(metrics[metric_name])
                    if total > 110:
                        self.config['xenfix'] = True
                        for mname in metrics.keys():
                            if '.idle' in mname:
                                metrics[mname] = float(metrics[mname]) / 2
                    elif total > 0:
                        self.config['xenfix'] = False
                else:
                    self.config['xenfix'] = False

            # Publish Metric Derivative
            for metric_name in metrics.keys():
                self.publish(metric_name,
                             metrics[metric_name])
            return True

        else:
            if not psutil:
                self.log.error('Unable to import psutil')
                self.log.error('No cpu metrics retrieved')
                return None

            cpu_time = psutil.cpu_times(True)
            total_time = psutil.cpu_times()
            for i in range(0, len(cpu_time)):
                metric_name = 'cpu' + str(i)
                self.publish(metric_name + '.user',
                             self.derivative(metric_name + '.user',
                                             cpu_time[i].user,
                                             self.MAX_VALUES['user']))
                if hasattr(cpu_time[i], 'nice'):
                    self.publish(metric_name + '.nice',
                                 self.derivative(metric_name + '.nice',
                                                 cpu_time[i].nice,
                                                 self.MAX_VALUES['nice']))
                self.publish(metric_name + '.system',
                             self.derivative(metric_name + '.system',
                                             cpu_time[i].system,
                                             self.MAX_VALUES['system']))
                self.publish(metric_name + '.idle',
                             self.derivative(metric_name + '.idle',
                                             cpu_time[i].idle,
                                             self.MAX_VALUES['idle']))

            metric_name = 'total'
            self.publish(metric_name + '.user',
                         self.derivative(metric_name + '.user',
                                         total_time.user,
                                         self.MAX_VALUES['user']))
            if hasattr(total_time, 'nice'):
                self.publish(metric_name + '.nice',
                             self.derivative(metric_name + '.nice',
                                             total_time.nice,
                                             self.MAX_VALUES['nice']))
            self.publish(metric_name + '.system',
                         self.derivative(metric_name + '.system',
                                         total_time.system,
                                         self.MAX_VALUES['system']))
            self.publish(metric_name + '.idle',
                         self.derivative(metric_name + '.idle',
                                         total_time.idle,
                                         self.MAX_VALUES['idle']))

            return True

        return None



#!/usr/bin/python
# coding=utf-8
################################################################################

from test import CollectorTestCase
from test import get_collector_config

from postgres import PostgresqlCollector


class TestPostgresqlCollector(CollectorTestCase):
    def setUp(self, allowed_names=None):
        if not allowed_names:
            allowed_names = []
        config = get_collector_config('PostgresqlCollector', {
        })
        self.collector = PostgresqlCollector(config, None)

    def test_import(self):
        self.assertTrue(PostgresqlCollector)



#!/usr/bin/python
# coding=utf-8
################################################################################

from test import CollectorTestCase
from test import get_collector_config
from test import unittest
from mock import Mock
from mock import patch

from diamond.collector import Collector
from exim import EximCollector

################################################################################


class TestEximCollector(CollectorTestCase):
    def setUp(self):
        config = get_collector_config('EximCollector', {
            'interval': 10,
            'bin': 'true'
        })

        self.collector = EximCollector(config, None)

    def test_import(self):
        self.assertTrue(EximCollector)

    @patch('os.access', Mock(return_value=True))
    @patch.object(Collector, 'publish')
    def test_should_work_with_synthetic_data(self, publish_mock):
        patch_communicate = patch('subprocess.Popen.communicate',
                                  Mock(return_value=('33', '')))

        patch_communicate.start()
        self.collector.collect()
        patch_communicate.stop()

        metrics = {
            'queuesize': 33.0
        }

        self.setDocExample(collector=self.collector.__class__.__name__,
                           metrics=metrics,
                           defaultpath=self.collector.config['path'])
        self.assertPublishedMany(publish_mock, metrics)

    @patch('os.access', Mock(return_value=True))
    @patch.object(Collector, 'publish')
    def test_should_fail_gracefully(self, publish_mock):
        patch_communicate = patch('subprocess.Popen.communicate',
                                  Mock(return_value=('', '')))

        patch_communicate.start()
        self.collector.collect()
        patch_communicate.stop()

        self.assertPublishedMany(publish_mock, {})

    @patch('os.access', Mock(return_value=False))
    @patch.object(Collector, 'publish')
    def test_should_fail_gracefully_2(self, publish_mock):
        self.collector.collect()
        self.assertPublishedMany(publish_mock, {})

################################################################################
if __name__ == "__main__":
    unittest.main()



# coding=utf-8

"""
Collect icmp round trip times
Only valid for ipv4 hosts currently

#### Dependencies

 * ping

#### Configuration

Configuration is done by:

Create a file named: PingCollector.conf in the collectors_config_path

 * enabled = true
 * interval = 60
 * target_1 = example.org
 * target_fw = 192.168.0.1
 * target_localhost = localhost

Test your configuration using the following command:

diamond-setup --print -C PingCollector

You should get a reponse back that indicates 'enabled': True and see entries
for your targets in pairs like:

'target_1': 'example.org'

We extract out the key after target_ and use it in the graphite node we push.

"""

import subprocess
import diamond.collector
import os
from diamond.collector import str_to_bool


class PingCollector(diamond.collector.Collector):

    def get_default_config_help(self):
        config_help = super(PingCollector, self).get_default_config_help()
        config_help.update({
            'bin':         'The path to the ping binary',
            'use_sudo':    'Use sudo?',
            'sudo_cmd':    'Path to sudo',
        })
        return config_help

    def get_default_config(self):
        """
        Returns the default collector settings
        """
        config = super(PingCollector, self).get_default_config()
        config.update({
            'path':             'ping',
            'bin':              '/bin/ping',
            'use_sudo':         False,
            'sudo_cmd':         '/usr/bin/sudo',
        })
        return config

    def collect(self):
        for key in self.config.keys():
            if key[:7] == "target_":
                host = self.config[key]
                metric_name = host.replace('.', '_')

                if not os.access(self.config['bin'], os.X_OK):
                    self.log.error("Path %s does not exist or is not executable"
                                   % self.config['bin'])
                    return

                command = [self.config['bin'], '-nq', '-c 1', host]

                if str_to_bool(self.config['use_sudo']):
                    command.insert(0, self.config['sudo_cmd'])

                ping = subprocess.Popen(
                    command, stdout=subprocess.PIPE).communicate()[0].strip(
                    ).split("\n")[-1]

                # Linux
                if ping.startswith('rtt'):
                    ping = ping.split()[3].split('/')[0]
                    metric_value = float(ping)
                # OS X
                elif ping.startswith('round-trip '):
                    ping = ping.split()[3].split('/')[0]
                    metric_value = float(ping)
                # Unknown
                else:
                    metric_value = 10000

                self.publish(metric_name, metric_value)



#!/usr/bin/python
# coding=utf-8
################################################################################

from test import CollectorTestCase
from test import get_collector_config
from test import unittest
from mock import Mock
from mock import patch

from diamond.collector import Collector
from sidekiqweb import SidekiqWebCollector

################################################################################


class TestSidekiqWebCollector(CollectorTestCase):
    def setUp(self):
        config = get_collector_config('SidekiqWebCollector', {
            'interval': 10
        })

        self.collector = SidekiqWebCollector(config, None)

    def test_import(self):
        self.assertTrue(SidekiqWebCollector)

    @patch.object(Collector, 'publish')
    def test_should_work_with_real_data(self, publish_mock):
        patch_urlopen = patch('urllib2.urlopen', Mock(
            return_value=self.getFixture('stats')))

        patch_urlopen.start()
        self.collector.collect()
        patch_urlopen.stop()

        metrics = {
            'redis.connected_clients': 22,
            'redis.uptime_in_days': 62,
            'redis.used_memory_human_byte': 1426063.36,
            'redis.used_memory_peak_human_byte': 8598323.2,
            'sidekiq.busy': 0,
            'sidekiq.default_latency': 0,
            'sidekiq.enqueued': 0,
            'sidekiq.failed': 22,
            'sidekiq.processed': 4622701,
            'sidekiq.retries': 0,
            'sidekiq.scheduled': 30,
        }

        self.assertPublishedMany(publish_mock, metrics)

    @patch.object(Collector, 'publish')
    def test_should_fail_gracefully(self, publish_mock):
        patch_urlopen = patch('urllib2.urlopen', Mock(
            return_value=self.getFixture('stats_blank')))

        patch_urlopen.start()
        self.collector.collect()
        patch_urlopen.stop()

        self.assertPublishedMany(publish_mock, {})

################################################################################
if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python
# coding=utf-8

###############################################################################

from test import CollectorTestCase
from test import get_collector_config
from test import unittest
from mock import patch

from diamond.collector import Collector
from onewire import OneWireCollector

###############################################################################


class TestOneWireCollector(CollectorTestCase):

    def setUp(self):
        config = get_collector_config('OneWireCollector', {
            'owfs': self.getFixturePath('.'),
            'scan': {'temperature': 't'},
            'id:28.2F702A010000': {'presure': 'p11'}})
        self.collector = OneWireCollector(config, None)

    def test_import(self):
        self.assertTrue(OneWireCollector)

    @patch.object(Collector, 'publish')
    def test(self, publish_mock):

        self.collector.collect()

        metrics = {
            '28_A76569020000.t': 22.4375,
            '28_2F702A010000.p11': 999
        }

        self.setDocExample(collector=self.collector.__class__.__name__,
                           metrics=metrics,
                           defaultpath=self.collector.config['path'])
        self.assertPublishedMany(publish_mock, metrics)


###############################################################################
if __name__ == "__main__":
    unittest.main()



# coding=utf-8

import re

_RE_FIND_FIRST_CAP = re.compile('(.)([A-Z][a-z]+)')
_RE_SPAN_OF_CAPS = re.compile('([a-z0-9])([A-Z])')


def camelcase_to_underscore(name):
    return _RE_SPAN_OF_CAPS.sub(r'\1_\2',
                                _RE_FIND_FIRST_CAP.sub(r'\1_\2', name)
                                ).lower()


class binary:
    """
    Store the value in bits so we can convert between things easily
    """
    value = None

    def __init__(self, value=None, unit=None):
        self.do(value=value, unit=unit)

    @staticmethod
    def convert(value=None, oldUnit=None, newUnit=None):
        convertor = binary(value=value, unit=oldUnit)
        return convertor.get(unit=newUnit)

    def set(self, value, unit=None):
        return self.do(value=value, unit=unit)

    def get(self, unit=None):
        return self.do(unit=unit)

    def do(self, value=None, unit=None):
        if not unit:
            return self.bit(value=value)

        if unit in ['bit', 'b']:
            return self.bit(value=value)
        if unit in ['kilobit', 'kbit', 'Kibit']:
            return self.kilobit(value=value)
        if unit in ['megabit', 'Mbit', 'Mibit', 'Mbit']:
            return self.megabit(value=value)
        if unit in ['gigabit', 'Gbit', 'Gibit']:
            return self.gigabit(value=value)
        if unit in ['terabit', 'Tbit', 'Tibit']:
            return self.terabit(value=value)
        if unit in ['petabit', 'Pbit', 'Pibit']:
            return self.petabit(value=value)
        if unit in ['exabit', 'Ebit', 'Eibit']:
            return self.exabit(value=value)
        if unit in ['zettabit', 'Zbit', 'Zibit']:
            return self.zettabit(value=value)
        if unit in ['yottabit', 'Ybit', 'Yibit']:
            return self.yottabit(value=value)

        if unit in ['byte', 'B']:
            return self.byte(value=value)
        if unit in ['kilobyte', 'kB', 'KiB']:
            return self.kilobyte(value=value)
        if unit in ['megabyte', 'MB', 'MiB', 'Mbyte']:
            return self.megabyte(value=value)
        if unit in ['gigabyte', 'GB', 'GiB']:
            return self.gigabyte(value=value)
        if unit in ['terabyte', 'TB', 'TiB']:
            return self.terabyte(value=value)
        if unit in ['petabyte', 'PB', 'PiB']:
            return self.petabyte(value=value)
        if unit in ['exabyte', 'EB', 'EiB']:
            return self.exabyte(value=value)
        if unit in ['zettabyte', 'ZB', 'ZiB']:
            return self.zettabyte(value=value)
        if unit in ['yottabyte', 'YB', 'YiB']:
            return self.yottabyte(value=value)

        raise NotImplementedError("unit %s" % unit)

    def bit(self, value=None):
        if value is None:
            return self.value
        else:
            self.value = float(value)

    def kilobit(self, value=None):
        if value is None:
            return self.bit() / 1024
        else:
            self.bit(value * 1024)

    def megabit(self, value=None):
        if value is None:
            return self.kilobit() / 1024
        else:
            self.kilobit(value * 1024)

    def gigabit(self, value=None):
        if value is None:
            return self.megabit() / 1024
        else:
            self.megabit(value * 1024)

    def terabit(self, value=None):
        if value is None:
            return self.gigabit() / 1024
        else:
            self.gigabit(value * 1024)

    def petabit(self, value=None):
        if value is None:
            return self.terabit() / 1024
        else:
            self.terabit(value * 1024)

    def exabit(self, value=None):
        if value is None:
            return self.petabit() / 1024
        else:
            self.petabit(value * 1024)

    def zettabit(self, value=None):
        if value is None:
            return self.exabit() / 1024
        else:
            self.exabit(value * 1024)

    def yottabit(self, value=None):
        if value is None:
            return self.zettabit() / 1024
        else:
            self.zettabit(value * 1024)

    def byte(self, value=None):
        if value is None:
            return self.value / 8
        else:
            self.value = float(value) * 8

    def kilobyte(self, value=None):
        if value is None:
            return self.byte() / 1024
        else:
            self.byte(value * 1024)

    def megabyte(self, value=None):
        if value is None:
            return self.kilobyte() / 1024
        else:
            self.kilobyte(value * 1024)

    def gigabyte(self, value=None):
        if value is None:
            return self.megabyte() / 1024
        else:
            self.megabyte(value * 1024)

    def terabyte(self, value=None):
        if value is None:
            return self.gigabyte() / 1024
        else:
            self.gigabyte(value * 1024)

    def petabyte(self, value=None):
        if value is None:
            return self.terabyte() / 1024
        else:
            self.terabyte(value * 1024)

    def exabyte(self, value=None):
        if value is None:
            return self.petabyte() / 1024
        else:
            self.petabyte(value * 1024)

    def zettabyte(self, value=None):
        if value is None:
            return self.exabyte() / 1024
        else:
            self.exabyte(value * 1024)

    def yottabyte(self, value=None):
        if value is None:
            return self.zettabyte() / 1024
        else:
            self.zettabyte(value * 1024)


class time:
    """
    Store the value in miliseconds so we can convert between things easily
    """
    value = None

    def __init__(self, value=None, unit=None):
        self.do(value=value, unit=unit)

    @staticmethod
    def convert(value=None, oldUnit=None, newUnit=None):
        convertor = time(value=value, unit=oldUnit)
        return convertor.get(unit=newUnit)

    def set(self, value, unit=None):
        return self.do(value=value, unit=unit)

    def get(self, unit=None):
        return self.do(unit=unit)

    def do(self, value=None, unit=None):
        if not unit:
            return self.millisecond(value=value)
        else:
            unit = unit.lower()

        if unit in ['millisecond', 'milliseconds', 'ms']:
            return self.millisecond(value=value)
        if unit in ['second', 'seconds', 's']:
            return self.second(value=value)

        raise NotImplementedError("unit %s" % unit)

    def millisecond(self, value=None):
        if value is None:
            return self.value
        else:
            self.value = float(value)

    def second(self, value=None):
        if value is None:
            return self.millisecond() / 1000
        else:
            self.millisecond(value * 1000)



# coding=utf-8

"""
Collects the complete status of most American Power Conversion Corp. (APC) UPSes
provided you have the apcupsd daemon installed, properly configured and
running. It can access status information from any APC UPS attached to the
localhost or attached to any computer on the network which is running
apcuspd in NIS mode.

#### Dependencies

 * apcuspd in NIS mode

"""

import diamond.collector
import socket
from struct import pack
import re
import time


class ApcupsdCollector(diamond.collector.Collector):

    def get_default_config_help(self):
        config_help = super(ApcupsdCollector, self).get_default_config_help()
        config_help.update({
            'hostname': 'Hostname to collect from',
            'port': 'port to collect from. defaults to 3551',
            'metrics': 'List of metrics. Valid metric keys can be found [here]'
            + '(http://www.apcupsd.com/manual/manual.html#status-report-fields)'
        })
        return config_help

    def get_default_config(self):
        """
        Returns the default collector settings
        """
        config = super(ApcupsdCollector, self).get_default_config()
        config.update({
            'path':     'apcupsd',
            'hostname': 'localhost',
            'port': 3551,
            'metrics': ['LINEV', 'LOADPCT', 'BCHARGE', 'TIMELEFT', 'BATTV',
                        'NUMXFERS', 'TONBATT', 'MAXLINEV', 'MINLINEV',
                        'OUTPUTV', 'ITEMP', 'LINEFREQ', 'CUMONBATT', ],
        })
        return config

    def getData(self):
        # Get the data via TCP stream
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.connect((self.config['hostname'], int(self.config['port'])))

        # Packet is pad byte, size byte, and command
        s.send(pack('xb6s', 6, 'status'))

        # Ditch the header
        s.recv(1024)
        time.sleep(.25)
        data = s.recv(4096)

        # We're done. Close the socket
        s.close()
        return data

    def collect(self):
        metrics = {}
        raw = {}

        data = self.getData()

        data = data.split('\n\x00')
        for d in data:
            matches = re.search("([A-Z]+)\s+:\s+(.*)$", d)
            if matches:
                value = matches.group(2).strip()
                raw[matches.group(1)] = matches.group(2).strip()
                vmatch = re.search("([0-9.]+)", value)
                if not vmatch:
                    continue
                try:
                    value = float(vmatch.group(1))
                except ValueError:
                    continue
                metrics[matches.group(1)] = value

        for metric in self.config['metrics']:
            if metric not in metrics:
                continue

            metric_name = "%s.%s" % (raw['UPSNAME'], metric)

            value = metrics[metric]

            if metric in ['TONBATT', 'CUMONBATT', 'NUMXFERS']:
                value = self.derivative(metric_name, metrics[metric])

            self.publish(metric_name, value)

        return True



#!/usr/bin/python
# coding=utf-8
################################################################################

from test import CollectorTestCase
from test import get_collector_config
from test import unittest
from test import run_only
from mock import patch

from diamond.collector import Collector
from puppetagent import PuppetAgentCollector

################################################################################


def run_only_if_yaml_is_available(func):
    try:
        import yaml
        yaml  # workaround for pyflakes issue #13
    except ImportError:
        yaml = None
    pred = lambda: yaml is not None
    return run_only(func, pred)


class TestPuppetAgentCollector(CollectorTestCase):
    def setUp(self):
        config = get_collector_config('PuppetAgentCollector', {
            'interval': 10,
            'yaml_path': self.getFixturePath('last_run_summary.yaml')
        })

        self.collector = PuppetAgentCollector(config, None)

    def test_import(self):
        self.assertTrue(PuppetAgentCollector)

    @run_only_if_yaml_is_available
    @patch.object(Collector, 'publish')
    def test(self, publish_mock):

        self.collector.collect()

        metrics = {
            'changes.total': 1,
            'events.failure': 0,
            'events.success': 1,
            'events.total': 1,
            'resources.changed': 1,
            'resources.failed': 0,
            'resources.failed_to_restart': 0,
            'resources.out_of_sync': 1,
            'resources.restarted': 0,
            'resources.scheduled': 0,
            'resources.skipped': 6,
            'resources.total': 439,
            'time.anchor': 0.009641,
            'time.augeas': 1.286514,
            'time.config_retrieval': 8.06442093849182,
            'time.cron': 0.00089,
            'time.exec': 9.780635,
            'time.file': 1.729348,
            'time.filebucket': 0.000633,
            'time.firewall': 0.007807,
            'time.group': 0.013421,
            'time.last_run': 1377125556,
            'time.mailalias': 0.000335,
            'time.mount': 0.002749,
            'time.package': 1.831337,
            'time.resources': 0.000371,
            'time.service': 0.734021,
            'time.ssh_authorized_key': 0.017625,
            'time.total': 23.5117989384918,
            'time.user': 0.02927,
            'version.config': 1377123965,
        }

        unpublished_metrics = {
            'version.puppet': '2.7.14',
        }

        self.setDocExample(collector=self.collector.__class__.__name__,
                           metrics=metrics,
                           defaultpath=self.collector.config['path'])

        self.assertPublishedMany(publish_mock, metrics)
        self.assertUnpublishedMany(publish_mock, unpublished_metrics)

################################################################################
if __name__ == "__main__":
    unittest.main()
